__author__ = 'mahajrod'

import numpy as np
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.ioff()
import matplotlib.ticker

from matplotlib.patches import Rectangle
from math import log

#TODO: rewrite for less memomory usage


class RecordGVF():
    def __init__(self, seqid, source, type, start, end, score, strand, phase, attributes_dict):
        self.seqid = seqid          #str
        self.source = source        #str
        self.type = type            #str
        self.start = start          #int
        self.end = end              #int
        self.score = score          #real or "."
        self.strand = strand        #"+" or "-"
        self.phase = phase          #str
        if ("ID" not in attributes_dict) or ("Variant_seq" not in attributes_dict) or ("Reference_seq" not in attributes_dict):
            raise ValueError("Attribute dictionary doesnt have one or more required keys ('ID', 'Variant_seq', 'Reference_seq')")
        self.attributes_dict = attributes_dict #dict

    def __str__(self):
        return self.string_form()

    def string_form(self):
        attributes_str = ';'.join([key + "=" + self.attributes_dict[key] for key in self.attributes_dict])
        return '\t'.join(map(lambda x: str(x), [self.seqid, self.source, self.type, self.start, self.end,
                                                self.score, self.strand, self.phase, attributes_str]))


class CollectionGVF():
    def __add_record(self, line):
        line_list = line.strip().split("\t")
        line_list[3] = int(line_list[3])
        line_list[4] = int(line_list[4])
        if line_list[5] != ".":
            line_list[5] = float(line_list[5])
        line_list[8] = {key: value for key, value in map(lambda x: x.split("="), line_list[8].split(";"))}
        self.records.append(RecordGVF(*line_list))

    def __add_metadata(self, line):
        tmp = line[2:].strip().split(" ")
        key, value = tmp[0], " ".join(tmp[1:])
        if key in self.metadata:
            self.metadata[key].append(value)
        else:
            self.metadata[key] = [value]

    def __init__(self, metadata=None, record_list=None, gvf_file=None, from_file=True):
        if not from_file:
            self.metadata = metadata
            self.records = record_list
        else:
            self.metadata = {}
            self.records = []
            with open(gvf_file, "r") as in_fd:
                for line in in_fd:
                    if line[0] != "#":
                        self.__add_record(line)
                        break
                    self.__add_metadata(line)
                for line in in_fd:
                    self.__add_record(line)
        self.longest_region_length = None
        if "sequence-region" in self.metadata:
            tmp_list = []
            for i in range(0, len(self.metadata["sequence-region"])):
                tmp = self.metadata["sequence-region"][i].split(" ")
                tmp_list.append((tmp[0], list(map(lambda x: int(x), tmp[1:]))))
            self.metadata["sequence-region"] = {key: value for key, value in tmp_list}

            self.longest_region_length = max([self.metadata["sequence-region"][region][1] -
                                              self.metadata["sequence-region"][region][0] + 1
                                              for region in self.metadata["sequence-region"]])

    def record_coordinates(self):
        #return dictionary, where keys are seqids and values numpy arrays of SNV coordinates
        sequence_varcoord_dict = {}
        for record in self.records:
            if record.seqid not in sequence_varcoord_dict:
                sequence_varcoord_dict[record.seqid] = [record.start]
            else:
                sequence_varcoord_dict[record.seqid].append(record.start)
        for seqid in sequence_varcoord_dict:
            sequence_varcoord_dict[seqid] = np.array(sequence_varcoord_dict[seqid])
        return sequence_varcoord_dict

    def stack_regions(self):
        #assume that records inside regions are sorted by start coordinate
        sequence_varcoord_dict = self.record_coordinates()
        staked = np.array([])
        if "sequence-region" in self.metadata:
            shift_dict = {}
            shift = 0
            for seqid in sorted(list(self.metadata["sequence-region"].keys())):
                shift_dict[seqid] = shift
                sequence_varcoord_dict[seqid] += shift - self.metadata["sequence-region"][seqid][0] + 1
                staked = np.hstack((staked, sequence_varcoord_dict[seqid]))
                length = self.metadata["sequence-region"][seqid][1] - \
                         self.metadata["sequence-region"][seqid][0] + 1
                shift += length
        else:
            #TODO:implement stacking for files without sequence-region metadata
            pass
        return {"All": staked}, shift_dict

    def rainfall_plot(self, output_file):
        #TODO: write rainfall plot
        pass

    def coverage_plot(self, cov_outfile="variation_coverage_plot.svg", distr_outfile="variation_distobution_plot.svg",
                      subplots=None,
                      cov_figsize=(10, 10), cov_dpi =300, cov_nbins=1000, cov_ylabel="N of SNV",
                      cov_xlabel="Position", share_x=False, share_y=True, single_plot=False,
                      distr_figsize=(10, 10), distr_dpi =300, distr_nbins=None, distr_ylabel="N of windows",
                      distr_xlabel="N of SNV in window",
                      black_list=[], white_list=[]):
        shift_dict = {}
        if single_plot:
            sequence_varcoord_dict, shift_dict = self.stack_regions()
        else:
            sequence_varcoord_dict = self.record_coordinates()
        num_of_seq = len(sequence_varcoord_dict)

        if (not subplots) or single_plot:
            subplot_tuple = (num_of_seq, 1)
        else:
            subplot_tuple = subplots

        #plt.figure(1, figsize=figsize, dpi=dpi)
        cov_fig, cov_axes = plt.subplots(nrows=subplot_tuple[0], ncols=subplot_tuple[1],
                                         figsize=cov_figsize, dpi=cov_dpi)
        #fig.suptitle('SNV coverage', fontsize=14, fontweight='bold')
        if not single_plot:
            cov_fig.tight_layout()
        else:
            cov_fig.suptitle('SNV coverage', fontweight='bold')
        index = 1
        subplot_list = []
        hist_dict = {}          #dict of n,bins,patches for each histogramm
        for seqid in sorted(list(sequence_varcoord_dict.keys())):
            if black_list and (seqid in black_list):
                continue
            if white_list and (seqid not in white_list):
                continue

            if subplot_list:
                subplot_list.append(plt.subplot(subplot_tuple[0], subplot_tuple[1], index, axisbg="#aaaaaa",
                                    sharey=subplot_list[0]))#, sharex=subplot_list[0]))
            else:
                subplot_list.append(plt.subplot(subplot_tuple[0], subplot_tuple[1], index, axisbg="#aaaaaa"))

            n_of_bins = cov_nbins
            if not single_plot:
                if "sequence-region" in self.metadata:
                    #plt.gca().add_patch(Rectangle((0, 0), self.metadata["sequence-region"][seqid][1], 900,facecolor="#aaaaaa", edgecolor='none'))
                    subplot_list[index-1].set_xlim(self.metadata["sequence-region"][seqid][0],
                                                   self.metadata["sequence-region"][seqid][1])

                if self.longest_region_length:
                    n_of_bins = 1 + int(cov_nbins * self.metadata["sequence-region"][seqid][1]/self.longest_region_length)
            else:
                last_region = sorted(list(self.metadata["sequence-region"].keys()))[-1]
                subplot_list[index-1].set_xlim(0, shift_dict[last_region] +
                                                  self.metadata["sequence-region"][last_region][1])

            hist_dict[seqid] = plt.hist(sequence_varcoord_dict[seqid], bins=n_of_bins, color="green")

            formatter = matplotlib.ticker.ScalarFormatter()
            formatter.set_powerlimits((-3, 2))
            subplot_list[index-1].xaxis.set_major_formatter(formatter)
            subplot_list[index-1].yaxis.set_major_formatter(formatter)
            plt.title("Chromosome %s" % seqid)
            #plt.legend(loc='upper right')
            plt.ylabel(cov_ylabel)
            #plt.xlabel(cov_xlabel)
            index += 1
        if shift_dict:
            #add chromosome edges
            for seqid in shift_dict:
                plt.plot((shift_dict[seqid], shift_dict[seqid]), (0, 400), 'k-', color="red")
            #plt.legend(loc=
        #print(hist_dict)
        plt.savefig(cov_outfile)
        plt.close()

        distr_fig, distr_axes = plt.subplots(nrows=subplot_tuple[0], ncols=subplot_tuple[1],
                                             figsize=distr_figsize, dpi=distr_dpi)
        if not single_plot:
            distr_fig.tight_layout()
        else:
            cov_fig.suptitle('SNV coverage distrobution', fontweight='bold')
        index = 1
        distr_subplot_list = []
        for seqid in sorted(list(hist_dict.keys())):
            """
            if black_list and (seqid in black_list):
                continue
            if white_list and (seqid not in white_list):
                continue
            """
            if distr_subplot_list:
                distr_subplot_list.append(plt.subplot(subplot_tuple[0], subplot_tuple[1], index, axisbg="#aaaaaa",
                                          sharey=subplot_list[0]))#, sharex=subplot_list[0]))
            else:
                distr_subplot_list.append(plt.subplot(subplot_tuple[0], subplot_tuple[1], index, axisbg="#aaaaaa"))

            n_of_bins = distr_nbins
            if not n_of_bins:
                n_of_bins = 1 + 3.32 * log(len(hist_dict[seqid][0]), 10)

            plt.hist(hist_dict[seqid][0], bins=n_of_bins, color="green")

            formatter = matplotlib.ticker.ScalarFormatter()
            formatter.set_powerlimits((-3, 2))
            subplot_list[index-1].xaxis.set_major_formatter(formatter)
            subplot_list[index-1].yaxis.set_major_formatter(formatter)
            plt.title("Chromosome %s" % seqid)
            #plt.legend(loc='upper right')
            plt.ylabel(distr_ylabel)
            plt.xlabel(distr_xlabel)
            #plt.xlabel(cov_xlabel)
            index += 1
        plt.savefig(distr_outfile)
        plt.close()

if __name__ == "__main__":
    gvf_col = CollectionGVF(gvf_file="/home/mahajrod/genetics/tetraodon/SNV/Tetraodon_nigroviridis.gvf", from_file=True)
    #print(gvf_col.metadata["sequence-region"])
    #gvf_col.coverage_plot(figsize=(30, 15), nbins=3000, subplots=(6, 5))#black_list=["Un_random", "1_random",
                                                                         #"15_random", "2_random",
                                                                         #"21_random"])
    gvf_col.coverage_plot(cov_outfile="variation_coverage_plot_single.svg",
                          distr_outfile="variation_distrobution_plot_single.svg",
                          cov_figsize=(22.5, 7.5), cov_nbins=3000,
                          distr_figsize=(10, 10), distr_nbins=50, single_plot=True)










#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool


class CombineGVCFs4(Tool):
    def __init__(self, max_threads=4, max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk CombineGVCFs",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)

    def parse_options(self, reference, gvcf_list, output, extension_list=["g.vcf",]):

        options = " -O %s" % output
        options += " -R %s" % reference

        for gvcf in self.make_list_of_path_to_files_by_extension(gvcf_list,
                                                                 extension_list=extension_list,
                                                                 recursive=False, return_absolute_paths=True):
            options += " --variant %s" % gvcf

        return options

    def combine(self, reference, gvcf_list, output, extension_list=["g.vcf",]):

        options = self.parse_options(reference,
                                     gvcf_list, output,
                                     extension_list=extension_list)

        self.execute(options=options)



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class TMAP(Tool):

    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "tmap", path=path, max_threads=max_threads)

    def index(self, reference):

        options = ""
        options += " -f %s" % reference

        self.execute(options, cmd="tmap index")

    def parse_global_options(self, reference, reads, output=None, reads_fmt=None, output_fmt=0,
                             aln_output_mode=None, verbose=False):

        """
        -i %s               Reads formats: fa, fasta, fastq, fq, sam, bam. Default auto detection
        -o %i               Output format:
                                0: sam
                                1: compressed bam
                                2: uncompressed bam
        -a %i               Output filter for the mappings:
                                0: returns the mapping with the best score only if all other mapping havd worse
                                   score, otherwise the read is unmapped
                                1: returns mapping with the best score. If more then one mapping has this score,
                                   a random mapping with this score is returned
                                2: returns all the mappings with the best score
                                3: returns all mappings, regardless of score
        """

        options = ""
        options += " -n %i" % self.max_threads
        options += " -f %s" % reference
        options += " -r %s" % reads
        options += " -i %s" % reads_fmt if reads_fmt else ""
        options += " -s %s" % output if output else ""
        options += " -o %i" % output_fmt
        options += " -v" if verbose else ""
        options += " -a %i" % aln_output_mode if aln_output_mode else ""
        return options

    def parse_options(self, *args, **kwargs):
        global_options_names = ["output", "reads_fmt", "output_fmt", "aln_output_mode", "verbose"]

        global_options_keys = []
        common_options_keys = []
        for key in kwargs.keys():
            if key in global_options_names:
                global_options_keys.append(key)
            else:
                common_options_keys.append(key)
        global_options = self.parse_global_options(*args, **dict([(key, kwargs[key]) for key in global_options_keys]))
        common_options = self.parse_common_mapping_options(**dict([(key, kwargs[key]) for key in common_options_keys]))

        return global_options, common_options

    @staticmethod
    def parse_common_mapping_options(match_score=None, mismatch_penalty=None, gap_open_pen=None,
                                     gap_ext_pen=None, gap_long_pen=None, gap_long_len=None,
                                     local_band_width=None, soft_clip_type=None, duplicate_window=None,
                                     max_seed_band=None, long_hit_mult=None, score_thres=None,
                                     vsw_type=None, reads_queue_size=None,
                                     sam_read_group=None, bidirectional=False, use_seq_equal=False,
                                     unroll_banding=False, rand_read_name=False, prefix_exclude=None,
                                     suffix_exclude=None, ignore_rg_from_sam=False, input_bz2=False,
                                     input_gz=False, end_repair=None, max_adapter_bases_for_soft_clipping=None,
                                     shared_memory_key=None, min_seq_length=None, max_seq_length=None):
        # TODO: add flowspace and pairing options
        """
        -g %i               Soft clip type:
                                0:
                                1:
                                2:
                                3:

        --end-repair        End repair:
                                0: disable
                                1: prefer mismatches
                                2: prefer indels
        """
        options = ""
        options += " -A %i" % match_score if match_score else ""
        options += " -M %i" % mismatch_penalty if mismatch_penalty else ""
        options += " -O %i" % gap_open_pen if gap_open_pen else ""
        options += " -E %i" % gap_ext_pen if gap_ext_pen else ""
        options += " -G %i" % gap_long_pen if gap_long_pen else ""
        options += " -K %i" % gap_long_len if gap_long_len else ""

        options += " -w %i" % local_band_width if local_band_width else ""
        options += " -g %i" % soft_clip_type if soft_clip_type else ""
        options += " -W %i" % duplicate_window if duplicate_window else ""
        options += " -B %i" % max_seed_band if max_seed_band else ""
        options += " --long-hit-mult %f" % long_hit_mult if long_hit_mult else ""
        options += " -T %i" % score_thres if score_thres else ""
        options += " -q %i" % reads_queue_size if reads_queue_size else ""
        options += " -H %i" % vsw_type if vsw_type else ""

        options += " -U" if unroll_banding else ""
        options += " -R %s" % sam_read_group if sam_read_group else ""
        options += " -D" if bidirectional else ""
        options += " -I" if use_seq_equal else ""
        options += " -u" if rand_read_name else ""
        options += " --prefix-exclude %i" % prefix_exclude if prefix_exclude else ""
        options += " --suffix-exclude %i" % suffix_exclude if suffix_exclude else ""
        options += " -C" if ignore_rg_from_sam else ""
        options += " -j" if input_bz2 else ""
        options += " -z" if input_gz else ""
        options += " --end-repair %i" % end_repair if end_repair else ""
        options += " -J %i" % max_adapter_bases_for_soft_clipping if max_adapter_bases_for_soft_clipping else ""
        options += " -k %i" % shared_memory_key if shared_memory_key else ""
        options += " --min_seq_length %i" % min_seq_length if min_seq_length else ""
        options += " --max_seq_length %i" % max_seq_length if max_seq_length else ""

        return options

    def map1(self, *args, **kwargs):
        # TODO: add parsing of  command specific options
        options = " ".join(self.parse_options(*args, **kwargs))
        self.execute(options, cmd="tmap map1")

    def map2(self, *args, **kwargs):
        # TODO: add parsing of  command specific options
        options = " ".join(self.parse_options(*args, **kwargs))
        self.execute(options, cmd="tmap map2")

    def map3(self, *args, **kwargs):
        # TODO: add parsing of  command specific options
        options = " ".join(self.parse_options(*args, **kwargs))
        self.execute(options, cmd="tmap map3")

    def map4(self, *args, **kwargs):
        # TODO: add parsing of  command specific options
        print(args, kwargs)
        options = " ".join(self.parse_options(*args, **kwargs))
        print(options)
        self.execute(options, cmd="tmap map4")

    def mapvsw(self, *args,  **kwargs):
        options = " ".join(self.parse_options(*args, **kwargs))
        self.execute(options, cmd="tmap mapvsw")

    def mapall(self, reference, reads, global_options_dict={}, map1_options_dict={}, map2_options_dict={}, map3_options_dict={}):
        options = "%s stage1 map1 %s  map2 %s map3 %s" % (self.parse_global_options(reference, reads,
                                                                                    **global_options_dict),
                                                          self.parse_common_mapping_options(**map1_options_dict),
                                                          self.parse_common_mapping_options(**map2_options_dict),
                                                          self.parse_common_mapping_options(**map3_options_dict))
        self.execute(options, cmd="tmap mapall")

    def fasta2pac(self):
        options = ""
        self.execute(options, cmd="tmap fasta2pac")

    def pac2bwt(self):
        options = ""
        self.execute(options, cmd="tmap pac2bwt")

    def bwt2sa(self):
        options = ""
        self.execute(options, cmd="tmap bwt2sa")

    def sff2fq(self):
        options = ""
        self.execute(options, cmd="tmap sff2fq")

    def sff2sam(self):
        options = ""
        self.execute(options, cmd="tmap sff2sam")

    def refinfo(self):
        options = ""
        self.execute(options, cmd="tmap refinfo")

    def pac2fasta(self):
        options = ""
        self.execute(options, cmd="tmap pac2fasta")

    def bwtupdate(self):
        options = ""
        self.execute(options, cmd="tmap bwtupdate")

    def indexsize(self):
        options = ""
        self.execute(options, cmd="tmap indexsize")

    def sam2fs(self):
        options = ""
        self.execute(options, cmd="tmap sam2fs")

    def sw(self):
        options = ""
        self.execute(options, cmd="tmap sw")


import os
import sys

if sys.version_info[0] == 3:
    from io import TextIOWrapper as file

from copy import deepcopy
from collections import OrderedDict
from RouToolPa.Collections.General import IdList, SynDict
from RouToolPa.Routines.Sequence import SequenceRoutines
from RouToolPa.Formats.VariantFormats import VCF_COLS

class VCFRoutines(SequenceRoutines):
    def __init__(self):
        SequenceRoutines.__init__(self)

    def combine_same_samples_vcfs(self, output, vcf_list=None,
                                  order_vcf_files=False, sort=False, chunk_folder=None, chunk_prefix=None,
                                  chunk_suffix=None, starting_chunk=None, chunk_number_list=None,
                                  close_fd_after=False, extension_list=[".vcf", ]):

        output_fd = output if isinstance(output, file) else open(output, "w")

        if vcf_list:
            vcf_files = self.make_list_of_path_to_files_by_extension(vcf_list, extension_list=extension_list,
                                                                     recursive=False, return_absolute_paths=True)

            #print(vcf_files)
            #print( type(vcf_files))
        else:
            pass

        if order_vcf_files:

            vcf_files.sort(key=self.natural_keys_int)

        if sort:
            unsorted_file = "%s.unsorted.tmp" % output

            string = "cat %s > %s" % (vcf_files[0], unsorted_file)
            print(string)
            os.system(string)
            #with open(unsorted_file, "w") as out_fd:
            #    pass

            for filename in vcf_files[1:]:
                string = "sed -n '/^[^#]/p' %s >> %s" % (filename, unsorted_file)
                print(string)
                os.system(string)

            sorting_string = "awk -F'\\t' '{if (substr($1,1,1) == \"#\") {print $0} else {print $0 | \"sort -k1,1 -k2,2n\"} }' %s > %s" % (unsorted_file, output)
            """
            sorting_string = "(sed '/^[^#]/Q' %s; sort -k1,1 -k2,2n %s) > %s" % (vcf_files[0],
                                                                                 unsorted_file,
                                                                                 output)
            """
            print(sorting_string)

            os.system(sorting_string)
            os.remove(unsorted_file)

        else:
            print("Handling file %s ..." % vcf_files[0])
            with open(vcf_files[0], "r") as in_fd:
                for line in in_fd:
                    output_fd.write(line)

            if len(vcf_files) > 1:
                for filename in vcf_files[1:]:
                    print("Handling file %s ..." % filename)
                    with open(filename, "r") as in_fd:
                        for line in in_fd:
                            if line[0] == "#":
                                continue
                            else:
                                output_fd.write(line)
                                break
                        for line in in_fd:
                            output_fd.write(line)

        if not isinstance(output, file) or close_fd_after:
            output_fd.close()

    def check_gvcf_integrity(self, gvcf_file, output_prefix, reference=None, length_dict=None, parsing_mode="parse"):
        len_dict = length_dict if length_dict else self.get_lengths(record_dict=self.parse_seq_file(reference,
                                                                                                    mode=parsing_mode),
                                                                    out_file=None,
                                                                    close_after_if_file_object=False)

        scaffold_dict = OrderedDict()

        with self.metaopen(gvcf_file, "r") as gvcf_fd:
            prev_scaffold = ""

            for line in gvcf_fd:
                #print line
                if line[0] == "#":
                    continue

                line_list = line.split("\t")
                scaffold = line_list[0]
                start = int(line_list[1])
                format = line_list[7].split(";")

                if (len(format) == 1) and (format[0][0:3] == "END"):
                    end = int(format[0].split("=")[1])
                else:
                    end = start + len(line_list[3]) - 1
                #print line_list
                #print scaffold, start, end, format

                if scaffold not in scaffold_dict:
                    scaffold_dict[scaffold] = []

                if scaffold != prev_scaffold:
                    scaffold_dict[scaffold].append([deepcopy(start), deepcopy(end)])
                else:
                    #print scaffold_dict[scaffold][-1][1]
                    if scaffold_dict[scaffold][-1][1] + 1 >= start:
                        scaffold_dict[scaffold][-1][1] = deepcopy(max(end, scaffold_dict[scaffold][-1][1]))
                    else:
                        print(scaffold_dict[scaffold])
                        print(line)
                        scaffold_dict[scaffold].append([deepcopy(start), deepcopy(end)])
                prev_scaffold = scaffold

        complete_scaffolds = IdList()
        fragmented_scaffolds = IdList()
        scaffolds_with_absent_fragments = IdList()

        with open("%s.scaffold_regions" % output_prefix, "w") as scaf_reg_fd:

            for scaffold in scaffold_dict:
                if len(scaffold_dict[scaffold]) > 1:
                    fragmented_scaffolds.append(scaffold)

                scaffold_length = sum(map(lambda s: s[1] - s[0] + 1, scaffold_dict[scaffold]))
                if scaffold_length != len_dict[scaffold]:
                    scaffolds_with_absent_fragments.append(scaffold)
                else:
                    complete_scaffolds.append(scaffold)
                scaf_reg_fd.write("%s\t%s\n" % (scaffold, ",".join(map(lambda s: "-".join(map(str,s)), scaffold_dict[scaffold]))))

        complete_scaffolds.write("%s.complete_scaffolds" % output_prefix)
        fragmented_scaffolds.write("%s.fragmented_scaffolds" % output_prefix)
        scaffolds_with_absent_fragments.write("%s.scaffolds_with_absent_fragments" % output_prefix)

    def extract_heterozygous_variants(self, input_vcf, output_prefix, mode="one", verbose=True):
        with self.metaopen(input_vcf, "r") as in_fd, \
             self.metaopen("%s.hetero.vcf" % output_prefix, "w") as het_fd, \
             self.metaopen("%s.homo.vcf" % output_prefix, "w") as homo_fd:

            het_counter = 0
            homo_counter =0

            for line in in_fd:
                if line[0] == "#":
                    het_fd.write(line)
                    homo_fd.write(line)
                else:
                    line_list = line.strip().split("\t")
                    info_list = line_list[VCF_COLS["FORMAT"]].split(":")
                    samples = map(lambda s: s.split(":"), line_list[9:])
                    for index in range(0, len(info_list)):
                        if info_list[index] == "GT":
                            genotype_index = index
                            break
                    heterozygous_sample_number = sum(map(lambda s: True if s[0] != s[1] else False,
                                                     map(lambda s: s[genotype_index].split("/"), samples)))

                    if mode == "one":
                        if heterozygous_sample_number > 0:
                            het_fd.write(line)
                            het_counter += 1
                        else:
                            homo_fd.write(line)
                            homo_counter += 1
                    elif mode == "all":
                        if heterozygous_sample_number == len(samples):
                            het_fd.write(line)
                            het_counter += 1
                        else:
                            homo_fd.write(line)
                            homo_counter += 1

            if verbose:
                print("Heterozygous variants: %i" % het_counter)
                print("Homozygous variants: %i" % homo_counter)
                print("Total variants: %i" % (het_counter + homo_counter))
    """ 
    @staticmethod
    def convert_gvcf_to_coverage_file(self, gvcf_file, coverage_file):
        with self.metaopen(coverage_file, "w") as out_fd:
            for line_list in self.file_line_as_list_generator(gvcf_file, comments_prefix="#", separator="\t"):
                scaffold = line
    """


#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import Tool


class GenomicsDBImport4(Tool):
    def __init__(self, max_threads=4, max_memory=None, timelog=None):
        Tool.__init__(self,
                      "gatk GenomicsDBImport",
                      max_threads=max_threads, max_memory=max_memory,
                      timelog=timelog)

    def parse_options(self, gvcf_list, output_dbi_dir, interval_list, extension_list=["g.vcf",]):

        options = " --genomicsdb-workspace-path %s" % output_dbi_dir
        options += " --intervals %s" % ",".join(interval_list)

        for gvcf in self.make_list_of_path_to_files_by_extension(gvcf_list,
                                                                 extension_list=extension_list,
                                                                 recursive=False, return_absolute_paths=True):
            options += " --variant %s" % gvcf

        return options

    def create_db(self, gvcf_list, output_dbi_dir, interval_list, extension_list=["g.vcf",]):

        options = self.parse_options(gvcf_list, output_dbi_dir,
                                     interval_list=interval_list,
                                     extension_list=extension_list)

        self.execute(options=options)



__author__ = 'mahajrod'
import os
from RouToolPa.Routines.SequenceCluster import SequenceClusterRoutines
from RouToolPa.Collections.General import SynDict, IdList


class GORoutines(SequenceClusterRoutines):
    def __init__(self):
        SequenceClusterRoutines.__init__(self)

    @staticmethod
    def extract_entries_by_GO_from_eggnogmapper_output(eggnogmapper_output, GO_file, output_prefix,
                                                       comments_prefix="#", separator="\t",
                                                       ):

        GO_list = IdList(filename=GO_file, column_number=0)

        #print "GOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOOO"
        #print GO_list
        print (len(GO_list))

        extracted_entries_file = "%s.annotations" % output_prefix

        extracted_entries = 0

        with open(eggnogmapper_output, "r") as eggnog_fd:
            with open(extracted_entries_file, "w") as out_fd:
                for line in eggnog_fd:
                    if line[0] == comments_prefix:
                        out_fd.write(line)
                        continue
                    line_list = line.strip().split(separator)
                    entry_GO_list = line_list[5].split(",")
                    #print entry_GO_list
                    for GO in entry_GO_list:
                        if GO in GO_list:
                            out_fd.write(line)
                            extracted_entries += 1
                            break

        print("Extracted %i entries" % extracted_entries)



__author__ = 'mahajrod'

from Data.Nucleotides import degenerated_nucleotides


class PrimerRoutines:
    
    def count_GC_AT_composition(self, primer_sequence):
        GC_count_min = 0
        GC_count_max = 0
        AT_count_min = 0
        AT_count_max = 0

        for nucleotide in str(primer_sequence).upper():
            if nucleotide == "G" or nucleotide == "C":
                GC_count_max += 1
                GC_count_min += 1
            elif nucleotide == "A" or nucleotide == "T" or nucleotide == "U":
                AT_count_max += 1
                AT_count_min += 1
            else:
                if nucleotide not in degenerated_nucleotides:
                    raise ValueError("Wrong nucleotide")
                else:
                    if nucleotide == "S":
                        GC_count_max += 1
                        GC_count_min += 1
                    elif nucleotide == "W":
                        AT_count_max += 1
                        AT_count_min += 1
                    else:
                        GC_count_max += 1
                        AT_count_max += 1
        return {"GC": [GC_count_min, GC_count_max],
                "AT": [AT_count_min, AT_count_max]}

    def melting_temperature_formula1(self, primer_sequence):
        # Tm = (wA+xT) * 2 + (yG+zC) * 4
        GC_AT_composition = self.count_GC_AT_composition(primer_sequence)

        max_temp = GC_AT_composition["GC"][1]*4 + GC_AT_composition["AT"][0]*2
        min_temp = GC_AT_composition["GC"][0]*4 + GC_AT_composition["AT"][1]*2

        return min_temp, max_temp

    def melting_temperature_formula2(self, primer_sequence):
        # Tm = 64.9 +41*(yG+zC-16.4)/(wA+xT+yG+zC)
        GC_AT_composition = self.count_GC_AT_composition(primer_sequence)

        max_temp = 64.9 + 41.0 * (float(GC_AT_composition["GC"][1]) - 16.4)/(float(len(primer_sequence)))
        min_temp = 64.9 + 41.0 * (float(GC_AT_composition["GC"][0]) - 16.4)/(float(len(primer_sequence)))

        return min_temp, max_temp



#!/usr/bin/env python

from RouToolPa.Tools.Abstract import Tool


class TransDecoder(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "TransDecoder", path=path, max_threads=max_threads)

    def extract_longest_orfs(self, transcripts_file, genetic_code=None, analyze_only_top_strand=None,
                             minimum_protein_length=100):
        """
        Possible genetic codes:
            universal (default)
            Euplotes
            Tetrahymena
            Candida
            Acetabularia
            Mitochondrial-Canonical
            Mitochondrial-Vertebrates
            Mitochondrial-Arthropods
            Mitochondrial-Echinoderms
            Mitochondrial-Molluscs
            Mitochondrial-Ascidians
            Mitochondrial-Nematodes
            Mitochondrial-Platyhelminths
            Mitochondrial-Yeasts
            Mitochondrial-Euascomycetes
            Mitochondrial-Protozoans
        """
        options = " -t %s" % transcripts_file
        options += " -G %s" % genetic_code if genetic_code else ""
        options += " -S" if analyze_only_top_strand else ""
        options += " -m %i" % minimum_protein_length

        self.execute(options, cmd="TransDecoder.LongOrfs")

    def predict_pep(self, transcripts_file, minimum_orf_length_if_no_other_evidence=None,
                    pfam_hits=None, blastp_hits=None,
                    file_with_orfs_for_training=None,
                    number_of_top_orfs_for_training=None):

        options = " -t %s" % transcripts_file
        options += " --retain_long_orfs %i" % minimum_orf_length_if_no_other_evidence \
            if minimum_orf_length_if_no_other_evidence else ""
        options += " --retain_pfam_hits %s" % pfam_hits if pfam_hits else ""
        options += " --retain_blastp_hits %s" % blastp_hits if blastp_hits else ""
        options += " --train %s" % file_with_orfs_for_training if file_with_orfs_for_training else ""
        options += " -T %i" % number_of_top_orfs_for_training if number_of_top_orfs_for_training else ""

        self.execute(options, cmd="TransDecoder.Predict")




    """
    def annotate_pep(self, transcripts_file, genetic_code=None, analyze_only_top_strand=None,
                     reuse_existing_files=None, minimum_protein_length=100, write_temp_to_current_dir=None):


        options = " --CPU %i" % self.threads
        options += " -t %s" % transcripts_file

        options += " -G %s" % genetic_code if genetic_code else ""
        options += " -S" if analyze_only_top_strand else ""
        options += " --reuse" if reuse_existing_files else ""
        options += " -m %i" % minimum_protein_length

        options += " --workdir" if write_temp__to_current_dir else ""
        options += " > %s" % output

        self.execute(options)

    """


__author__ = 'mahajrod'
import os
from collections import Iterable, OrderedDict
from math import sqrt
#import matplotlib

#matplotlib.use('Agg')
import matplotlib.pyplot as plt
plt.ioff()
import numpy as np

from RouToolPa.Collections.General import TwoLvlDict

built_in_flags = {"DA": "desaminase-like",
                  "BR": "location in bad region (masked and so on)",
                  "IP": "indel presence"
                  }


class Record():
    def __init__(self, chrom, pos, description=None, flags=None):
        self.chrom = chrom
        self.pos = pos
        self.description = description
        self.flags = flags

    def __str__(self):
        pass

    @staticmethod
    def get_synonym(name, use_synonym=False, synonym_dict=None):
        if (not use_synonym) or (not synonym_dict):
            return name

        if name not in synonym_dict:
            return name
        return synonym_dict[name]

    def get_location(self, record_dict, key="Loc", strand_key="strand", feature_type_black_list=[],
                    use_synonym=False, synonym_dict=None):
        # function is written for old variant (with sub_feature)s rather then new (with CompoundLocation)
        # id of one SeqRecord in record_dict must be equal to record.pos
        # locations will be written to description dictionary of record using "key" as key 
        if not self.description:
            self.description = {}

        if key not in self.description:
            self.description[key] = set([])

        if strand_key not in self.description:
            # by default
            self.description[strand_key] = None
        #print(self.chrom, self.pos)
        for feature in record_dict[self.chrom].features:
            if (self.pos - 1) in feature:
                self.description[key].add(self.get_synonym(feature.type, use_synonym=use_synonym,
                                                           synonym_dict=synonym_dict))
                if self.description[strand_key] is None:
                    self.description[strand_key] = feature.strand
                elif feature.strand != self.description[strand_key]:
                    self.description[strand_key] = 0

                #print(feature)
            for sub_feature in feature.sub_features:
                if (self.pos - 1) in sub_feature:
                    self.description[key].add(self.get_synonym(sub_feature.type, use_synonym=use_synonym,
                                                               synonym_dict=synonym_dict))
                    if self.description[strand_key] is None:
                        self.description[strand_key] = sub_feature.strand
                    elif sub_feature.strand != self.description[strand_key]:
                        self.description[strand_key] = 0

        if not self.description[key]:
            # igc == intergenic
            self.description[key].add("igc")

    def check_location(self, bad_region_collection_gff, expression="bad_region.start <= self.pos <= bad_region.end"):
        if not self.flags:
            self.flags = set()
        for bad_region in bad_region_collection_gff:
            if self.chrom != bad_region.chrom:
                continue
            if eval(expression):
                self.flags.add("BR")


class Metadata():
    def __init__(self, metadata=[]):
        self.metadata = metadata

    def add_metadata(self):
        pass

    def __str__(self):
        pass


class Header():
    def __init__(self):
        pass

    def __str__(self):
        pass


class Collection(Iterable):
    # TODO: develop this class to minimize new code in various collections
    def __init__(self, metadata=None, record_list=None, header=None, input_file=None, from_file=False):
        # metadata should be Metadata class
        # header should be Header class
        # record_list should be list of Record class
        if from_file:
            self.read(input_file)
        else:
            self.metadata = metadata
            self.records = record_list
            self.header = header

    def read(self, input_file):
        # collectiontype-dependent function
        pass

    def add_metadata(self):
        # collectiontype-dependent function
        pass

    def add_header(self):
        # collectiontype-dependent function
        pass

    def add_record(self):
        # collectiontype-dependent function
        pass

    def __str__(self):
        collection_string = ""
        if self.metadata:
            collection_string += str(self.metadata)
        if self.header:
            collection_string += "\n" + str(self.header)
        if self.records:
            collection_string += "\n" + "\n".join([str(record) for record in self.records])

        return collection_string

    def __iter__(self):
        for record in self.records:
            yield record

    def __getitem__(self, item):
        #string-like access
        return self.records[item]

    def __len__(self):
        return len(self.records)

    def pop(self, index=None):
        # i am a fairy idiot - forgot that 0 is equal to False in python.
        # What to do? Everywhere make direct check if variable is None
        return self.records.pop() if index is None else self.records.pop(index)

    def filter_records_by_parameter_value(self, parameter, minimum=None, maximum=None):
        # TODO: check
        filtered_records = []
        filtered_out_records = []
        if (minimum is not None) and (maximum is not None):
            comparison = lambda par, min_value, max_value: min_value <= par <= max_value
        elif (minimum is None) and (maximum is None):
            raise ValueError("Both minimum and maximum thresholds were not set")
        elif minimum is None:
            comparison = lambda par, min_value, max_value: par <= max_value
        else:
            comparison = lambda par, min_value, max_value: min_value <= par

        for record in self.records:
            if comparison(getattr(record, parameter), minimum, maximum):
                filtered_records.append(record)
            else:
                filtered_out_records.append(record)

        return filtered_records, filtered_out_records

    def filter_records_by_expression(self, expression):
        filtered_records = []
        filtered_out_records = []
        for record in self.records:
            if eval(expression):
                filtered_records.append(record)
            else:
                filtered_out_records.append(record)

        return filtered_records, filtered_out_records

    def filter_records_by_function(self, function):
        filtered_records = []
        filtered_out_records = []
        for record in self.records:
            #print("a\na\na\na\na\na\na\na")
            #print(record.description["Power"])
            if function(record):
                filtered_records.append(record)
            else:
                filtered_out_records.append(record)

        return filtered_records, filtered_out_records

    def check_location(self, bad_region_collection_gff, expression="bad_region.start <= self.pos <= bad_region.end"):
        for record in self:
            record.check_location(bad_region_collection_gff, expression=expression)
    """
    def filter_by_expression(self, expression):
        self_type = self.__class__.__name__
        filtered_records, filtered_out_records = self.filter_records_by_expression(expression)
        #TODO: think how to rewrite following damned string!!!!!!!!!!
        exec("from Parsers.%s import %s" % (self_type[10:], self_type))
        return eval(self_type)(metadata=self.metadata, record_list=filtered_records,
                               header_list=self.header_list, from_file=False), \
               eval(self_type)(metadata=self.metadata, record_list=filtered_out_records,
                               header_list=self.header_list, from_file=False)
    """
    def split_records_by_flags(self, flag_set, mode="all"):
        # possible modes:
        # all - record to be counted as 'with flag' must have all flags from flags_list
        # one - record to be counted as 'with flag' must have at least one flag from flags_list
        flags = set(flag_set)
        with_flags_records = []
        without_flags_records = []
        if mode == "all":
            for record in self:
                if flags & record.flags == flags:
                    with_flags_records.append(record)
                else:
                    without_flags_records.append(record)
        elif mode == "one":
            for record in self:
                if flags & record.flags:
                    with_flags_records.append(record)
                else:
                    without_flags_records.append(record)

        return with_flags_records, without_flags_records

    def write(self, output_file):
        with open(output_file, "w") as out_fd:
            if self.metadata:
                out_fd.write(str(self.metadata) + "\n")
            if self.header:
                out_fd.write(str(self.header) + "\n")
            for record in self.records:
                out_fd.write(str(record) + "\n")

    def get_location(self, record_dict, key="Loc", use_synonym=False, synonym_dict=None):
        for record in self.records:
            record.get_location(record_dict, key=key, use_synonym=use_synonym, synonym_dict=synonym_dict)

    def _split_regions(self):
        splited_dict = OrderedDict({})
        for record in self.records:
            if record.chrom not in splited_dict:
                splited_dict[record.chrom] = [record]
            else:
                splited_dict[record.chrom].append(record)
        return splited_dict

    def count_locations(self, annotation_black_list=[],
                        allow_several_counts_of_record=False,
                        out_filename="location_counts.t",
                        write=True,
                        count_dir="location_counts"):
        os.system("mkdir -p %s" % count_dir)
        regions_dict = self._split_regions()
        region_counts_dict = TwoLvlDict({})
        for region in regions_dict:
            count_locations_dict = {"igc": 0}

            for record in regions_dict[region]:
                if (not record.description["Loc"]) or ("Loc" not in record.description):
                    count_locations_dict["unknown"] += 1
                    continue
                #print(record.description["Loc"])
                if allow_several_counts_of_record:
                    for location in record.description["Loc"]:
                        if location in annotation_black_list:
                            continue
                        if location not in count_locations_dict:
                            count_locations_dict[location] = 1
                        else:
                            count_locations_dict[location] += 1
                else:
                    full_location = []
                    for location in record.description["Loc"]:
                        if location in annotation_black_list:
                            continue
                        full_location.append(location)
                    if not full_location:
                        continue
                    full_location.sort()
                    full_location = "/".join(full_location)
                    if full_location not in count_locations_dict:
                        count_locations_dict[full_location] = 1
                    else:
                        count_locations_dict[full_location] += 1

            labels = []
            counts = []
            #colors = []
            for location in count_locations_dict:
                if count_locations_dict[location] == 0 or location in annotation_black_list:
                    continue
                labels.append(location)
                counts.append(count_locations_dict[location])
            region_counts_dict[region] = OrderedDict([(label, count) for label, count in zip(labels, counts)])

        if write:
            region_counts_dict.write("%s/%s" % (count_dir, out_filename))
        return region_counts_dict

    def location_pie(self, pie_name="Location of variants", annotation_colors=[],
                     dpi=150, figsize=(30, 30), facecolor="#D6D6D6",
                     ref_genome=None, explode=True, annotation_black_list=[],
                     allow_several_counts_of_record=False,
                     pie_filename="variant_location_pie.svg",
                     full_genome_pie_filename="variant_location_pie_full_genome.svg",
                     counts_filename="location_counts.t",
                     plot_dir="variant_location_pie",
                     counts_dir="location_counts",
                     radius = 1):
        print("Drawing location pie...")

        os.system("mkdir -p %s" % plot_dir)
        reference_colors = {"CDS": "#FBFD2B",    # yellow
                            "5'_UTR": "#FF000F",
                            "five_prime_UTR": "#FF000F",     # red
                            "3'_UTR": "#000FFF",
                            "three_prime_UTR": "#000FFF",     # blue
                            "igc": "#4ED53F",     # green
                            "ncRNA": 'cyan',
                            "other": "#ADB696"
                            }

        if annotation_colors:
            reference_colors = annotation_colors
            reference_colors["other"] = "#ADB696"

        count_locations_dict = self.count_locations(annotation_black_list=annotation_black_list,
                                                    allow_several_counts_of_record=allow_several_counts_of_record,
                                                    out_filename=counts_filename,
                                                    write=True,
                                                    count_dir=counts_dir)
        num_of_regions = len(count_locations_dict)
        side = int(sqrt(num_of_regions))
        if side*side != num_of_regions:
            side += 1
        sub_plot_dict = OrderedDict({})
        fig = plt.figure(2, dpi=dpi, figsize=figsize)
        fig.suptitle(pie_name, fontsize=20)

        index = 1
        all_labels = []
        all_counts = []
        all_colors = []

        for region in count_locations_dict:
            sub_plot_dict[region] = plt.subplot(side, side, index, axisbg="#D6D6D6")

            labels = []
            counts = []
            colors = []
            #print(count_locations_dict)
            for label in count_locations_dict[region]:
                #print(count_locations_dict[region])
                if count_locations_dict[region][label] == 0 or label in annotation_black_list:
                    continue
                labels.append(label)
                counts.append(count_locations_dict[region][label])
                if label not in reference_colors:
                    colors.append(reference_colors["other"])
                else:
                    colors.append(reference_colors[label])

            explodes = np.zeros(len(counts))
            if explode and counts:
                max_count_index = counts.index(max(counts))
                explodes[max_count_index] = 0.1
            plt.pie(counts, explode=explodes, labels=labels, colors=colors,
                    autopct='%1.1f%%', shadow=True, startangle=90)
            plt.title("Region %s" % region, y=1.08)
            # Set aspect ratio to be equal so that pie is drawn as a circle.
            plt.axis('equal')
            index += 1
            for i in range(0, len(labels)):
                if labels[i] not in all_labels:
                    all_labels.append(labels[i])
                    all_counts.append(counts[i])
                    all_colors.append(colors[i])
                else:
                    label_index = all_labels.index(labels[i])
                    all_counts[label_index] += counts[i]

        plt.savefig("%s/%s" % (plot_dir, pie_filename))
        plt.close()

        fig = plt.figure(3, dpi=dpi, figsize=(6, 6))
        fig.suptitle(pie_name, fontsize=20)
        plt.subplot(1, 1, 1, axisbg="#D6D6D6")
        all_explodes = np.zeros(len(all_counts))
        if explode and all_counts:
            max_count_index = all_counts.index(max(all_counts))
            all_explodes[max_count_index] = 0.1
        if len(all_labels) > 0:
            max_label_length = max([len(x) for x in all_labels])
            max_count = max(all_counts)
            max_letters = 1
            while int(max_count / 10**max_letters) != 0:
                max_letters += 1

        patches, texts = plt.pie(all_counts, explode=all_explodes, colors=all_colors,
                                            shadow=True, startangle=90, radius=2) #labels=all_labels,

        #patches, texts, autotexts = plt.pie(all_counts, explode=all_explodes, colors=all_colors,
        #                                    shadow=True, startangle=90, autopct='%1.1f%%', radius=4) #labels=all_labels

        #colors = ['yellowgreen','red','gold','lightskyblue','white','lightcoral','blue','pink', 'darkgreen','yellow','grey','violet','magenta','cyan']
        porcent = 100 * np.array(all_counts).astype(np.float32, copy=False)/sum(all_counts)

        labels = ['{0}  -  {1} ({2:1.2f}%)'.format(i.ljust(max_label_length), str(j).ljust(max_letters), k) for i, j, k in zip(all_labels, all_counts, porcent)]

        if len(all_labels) > 0:
            sort_legend = True
            if sort_legend:
                patches, labels, dummy = zip(*sorted(zip(patches, labels, all_counts),
                                                     key=lambda x: x[2],
                                                     reverse=True))

        plt.legend(patches, labels, loc='center left',  fontsize=13, bbox_to_anchor=(-1.0, 0.5)) # bbox_to_anchor=(-0.1, 1.),
        plt.title("Full genome")
            # Set aspect ratio to be equal so that pie is drawn as a circle.
        plt.axis('equal')
        plt.savefig("%s/%s" % (plot_dir, full_genome_pie_filename), bbox_inches='tight')
        plt.close()


class Parser:
    @staticmethod
    def get_filtered_entry_list(entry_list,
                                entry_black_list=[],
                                sort_entries=False,
                                entry_ordered_list=None,
                                entry_white_list=[]):
        white_set = set(entry_white_list)
        black_set = set(entry_black_list)
        entry_set = set(entry_list)

        if white_set:
            entry_set = entry_set & white_set
        if black_set:
            entry_set = entry_set - black_set

        filtered_entry_list = list(entry_set)
        if sort_entries:
            filtered_entry_list.sort()

        final_entry_list = []

        if entry_ordered_list:
            for entry in entry_ordered_list:
                if entry in filtered_entry_list:
                    final_entry_list.append(entry)
                    filtered_entry_list.remove(entry)
                else:
                    print("WARNING!!!Entry(%s) from order list is absent in list of entries!" % entry)
            return final_entry_list + filtered_entry_list
        else:
            return filtered_entry_list

if __name__ == "__main__":
    fg = Collection()


#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class ToolTemplate(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "tool_command", path=path, max_threads=max_threads)

    def tool_method(self, tool_options):
        options = ""
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        options += " "
        self.execute(options, cmd="custom command if necessary ex: tool_command subcommand")


if __name__ == "__main__":
    pass


__author__ = 'mahajrod'

from RouToolPa.Tools.Alignment.BLAT import BLAT
from RouToolPa.Tools.Alignment.GEM import GEM
from RouToolPa.Tools.Alignment.BWA import BWA
from RouToolPa.Tools.Alignment.TMAP import TMAP
from RouToolPa.Tools.Alignment.STAR import STAR
from RouToolPa.Tools.Alignment.Tophat import Tophat
from RouToolPa.Tools.Alignment.Bowtie2 import Bowtie2
from RouToolPa.Tools.Alignment.Novoalign import Novoalign
from RouToolPa.Tools.Alignment.LongRanger import LongRanger


max_threads = 4
bowtie2_path = ""
bwa_path = ""
novoalign_path = ""
tmap_path = ""
prank_path = ""

GEM = GEM()
BWA = BWA(path=bwa_path, max_threads=max_threads)
TMAP = TMAP(path=novoalign_path, max_threads=max_threads)
BLAT = BLAT()
STAR = STAR()
Tophat = Tophat()
Bowtie2 = Bowtie2(path=bowtie2_path, max_threads=max_threads)
Novoalign = Novoalign(path=novoalign_path, max_threads=max_threads)
LongRanger = LongRanger()



#!/usr/bin/env python
import os
from math import log, floor
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Routines import FileRoutines
from RouToolPa.Tools.Samtools import SamtoolsV1

class STAR(Tool):
    """
    STAR doesn't understand local paths that is why os.path.abspath is used everywhere
    """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "STAR", path=path, max_threads=max_threads)

    # TODO: methods below are temporarly! Remove them when switch to RouToolPa will be finished
    @staticmethod
    def add_external_extraction_to_filename(filename):
        if filename[-3:] == ".gz":
            return "<(gunzip -c %s)" % filename
        elif filename[-4:] == ".bz2":
            return "<(bunzip2 -c %s)" % filename
        else:
            return filename

    @staticmethod
    def add_external_extraction_to_filelist(filelist):
        new_filelist = []

        for filename in filelist:
            if filename[-3:] == ".gz":
                new_filelist.append("<( gunzip -c %s)" % filename)
            elif filename[-4:] == ".bz2":
                new_filelist.append("<(bunzip2 -c %s)" % filename)
            else:
                new_filelist.append(filename)

        return new_filelist

    def index(self, genome_dir, genome_fasta, annotation_gtf=None, feature_from_gtf_to_use_as_exon=None, junction_tab_file=None, sjdboverhang=None,
              genomeSAindexNbases=None, genomeChrBinNbits=None, genome_size=None):

        FileRoutines.safe_mkdir(genome_dir)

        options = "--runMode genomeGenerate"
        options += " --genomeDir %s" % os.path.abspath(genome_dir)
        options += " --runThreadN %i" % self.threads
        options += " --genomeFastaFiles %s" % (os.path.abspath(genome_fasta) if isinstance(genome_fasta, str) else " ".join(map(os.path.abspath, genome_fasta)))
        options += " --sjdbGTFfile %s" % annotation_gtf if annotation_gtf else ""
        options += " --sjdbFileChrStartEnd %s" % junction_tab_file if junction_tab_file else ""
        options += " --sjdbOverhang %i" % sjdboverhang if sjdboverhang else "" # number of bases taken from both sides of splice junction. 100 by default
        if genome_size:
            options += " --genomeSAindexNbases %i" % min([14, (floor(log(genome_size, 2)/2)) - 1])
        else:
            options += " --genomeSAindexNbases %i" % genomeSAindexNbases if genomeSAindexNbases else "" # size of k-mers used for preindexing of suffix array
        options += " --genomeChrBinNbits %i" % genomeChrBinNbits if genomeChrBinNbits else "" # padding size (log2) of reference sequences. 18 by default
        # recommended value min(18, log2(GenomeLength/NumberOfScaffolds))
        options += " --sjdbGTFfeatureExon %s" % feature_from_gtf_to_use_as_exon if feature_from_gtf_to_use_as_exon else ""
        self.execute(options)

    def align(self, genome_dir, forward_read_list, reverse_read_list=None, annotation_gtf=None,
              feature_from_gtf_to_use_as_exon=None, exon_tag_to_use_as_transcript_id=None,
              exon_tag_to_use_as_gene_id=None, length_of_sequences_flanking_junction=None, junction_tab_file_list=None,
              three_prime_trim=None, five_prime_trim=None, adapter_seq_for_three_prime_clip=None,
              max_mismatch_percent_for_adapter_trimming=None, three_prime_trim_after_adapter_clip=None,
              output_type="BAM", sort_bam=True, max_memory_per_thread_for_bam_sorting="4G", include_unmapped_reads_in_bam=True,
              output_unmapped_reads=True, output_dir="./", two_pass_mode=False, max_intron_length=None):
        if reverse_read_list:
            if len(forward_read_list) != len(reverse_read_list):
                raise ValueError("Wrong read file pairing")

        options = " --runThreadN %i" % self.threads
        options += " --genomeDir %s" % os.path.abspath(genome_dir)
        options += " --sjdbGTFfile %s" % annotation_gtf if annotation_gtf else ""
        options += " --sjdbGTFtagExonParentTranscript %s" % exon_tag_to_use_as_transcript_id if exon_tag_to_use_as_transcript_id else ""
        options += " --sjdbGTFtagExonParentGene %s" % exon_tag_to_use_as_gene_id if exon_tag_to_use_as_gene_id else ""
        options += " --sjdbGTFfeatureExon %s" % feature_from_gtf_to_use_as_exon if feature_from_gtf_to_use_as_exon else ""

        options += " --sjdbOverhang %i" % length_of_sequences_flanking_junction if length_of_sequences_flanking_junction else ""
        options += (" --sjdbFileChrStartEnd %s" % (os.path.abspath(junction_tab_file_list) if isinstance(junction_tab_file_list, str) else " ".join(map(os.path.abspath, junction_tab_file_list)))) if junction_tab_file_list else ""

        forward_read_abs_path_list = [os.path.abspath(forward_read_list)] if isinstance(forward_read_list, str) else map(os.path.abspath, reverse_read_list)
        reverse_read_abs_path_list = ([os.path.abspath(reverse_read_list)] if isinstance(reverse_read_list, str) else map(os.path.abspath, reverse_read_list)) if reverse_read_list else None

        forward_read_abs_path_list = self.add_external_extraction_to_filelist(forward_read_abs_path_list)
        reverse_read_abs_path_list = self.add_external_extraction_to_filelist(reverse_read_abs_path_list)

        options += " --readFilesIn %s" % " ".join(forward_read_abs_path_list)

        options += " %s" % " ".join(reverse_read_abs_path_list) if reverse_read_abs_path_list else ""

        options += " --clip3pNbases %i" % three_prime_trim if three_prime_trim else ""
        options += " --clip5pNbases %i" % five_prime_trim if five_prime_trim else ""
        options += " --clip3pAdapterSeq %s" % adapter_seq_for_three_prime_clip if adapter_seq_for_three_prime_clip else ""
        options += " --clip3pAdapterMMp %f" % max_mismatch_percent_for_adapter_trimming if max_mismatch_percent_for_adapter_trimming else ""
        options += " --clip3pAfterAdapterNbases %i" % three_prime_trim_after_adapter_clip if three_prime_trim_after_adapter_clip else ""

        options += " --outSAMtype %s %s" % (output_type, "Unsorted") # "SortedByCoordinate" if sort_bam else "Unsorted")
        #options += " --limitBAMsortRAM %i" % max_memory_for_bam_sorting if max_memory_for_bam_sorting else ""
        options += " --outSAMunmapped Within" if include_unmapped_reads_in_bam else ""
        options += " --outReadsUnmapped Fastx" if output_unmapped_reads else ""
        options += " --outFileNamePrefix %s" % output_dir if output_dir else ""
        options += " --twopassMode Basic" if two_pass_mode else ""
        options += " --alignIntronMax %i" % max_intron_length if max_intron_length else ""

        self.execute(options)

        if sort_bam:
            print("\tSorting...")
            unsorted_bam = "%s/Aligned.out.bam" % output_dir
            sorted_bam = "%s/Aligned.sortedByCoord.out.bam" % output_dir
            SamtoolsV1.threads = self.threads
            SamtoolsV1.sort(unsorted_bam, sorted_bam, max_memory_per_thread=max_memory_per_thread_for_bam_sorting)

            print("\tIndexing bam file...")
            SamtoolsV1.index(sorted_bam)

    def align_samples(self, samples_dir, output_dir, genome_dir, genome_fasta=None, samples=None,
                      annotation_gtf=None,  sjdboverhang=None,
                      genomeSAindexNbases=None, genomeChrBinNbits=None, genome_size=None,
                      feature_from_gtf_to_use_as_exon=None, exon_tag_to_use_as_transcript_id=None,
                      exon_tag_to_use_as_gene_id=None, length_of_sequences_flanking_junction=None,
                      junction_tab_file_list=None, three_prime_trim=None, five_prime_trim=None,
                      adapter_seq_for_three_prime_clip=None, max_mismatch_percent_for_adapter_trimming=None,
                      three_prime_trim_after_adapter_clip=None, output_type="BAM", sort_bam=True,
                      max_memory_per_thread_for_bam_sorting="4G", include_unmapped_reads_in_bam=True,
                      output_unmapped_reads=True, two_pass_mode=True, max_intron_length=None):
        #STAR.threads = threads
        #STAR.path = star_dir
        
        if genome_fasta:
            STAR.index(genome_dir, genome_fasta, annotation_gtf=annotation_gtf,
                       junction_tab_file=junction_tab_file_list, sjdboverhang=sjdboverhang,
                       genomeSAindexNbases=genomeSAindexNbases, genomeChrBinNbits=genomeChrBinNbits,
                       genome_size=genome_size)
        
        sample_list = samples if samples else self.get_sample_list(samples_dir)
        
        FileRoutines.safe_mkdir(output_dir)
        
        for sample in sample_list:
            print ("Handling %s" % sample)
            sample_dir = "%s/%s/" % (samples_dir, sample)
            alignment_sample_dir = "%s/%s/" % (output_dir, sample)
            FileRoutines.safe_mkdir(alignment_sample_dir)
            filetypes, forward_files, reverse_files, se_files = FileRoutines.make_lists_forward_and_reverse_files(sample_dir)
        
            print("\tAligning reads...")
        
            self.align(genome_dir, forward_files, reverse_read_list=reverse_files,
                       annotation_gtf=annotation_gtf if not genome_fasta else None,
                       feature_from_gtf_to_use_as_exon=feature_from_gtf_to_use_as_exon,
                       exon_tag_to_use_as_transcript_id=exon_tag_to_use_as_transcript_id,
                       exon_tag_to_use_as_gene_id=exon_tag_to_use_as_gene_id,
                       length_of_sequences_flanking_junction=length_of_sequences_flanking_junction,
                       junction_tab_file_list=junction_tab_file_list,
                       three_prime_trim=three_prime_trim, five_prime_trim=five_prime_trim,
                       adapter_seq_for_three_prime_clip=adapter_seq_for_three_prime_clip,
                       max_mismatch_percent_for_adapter_trimming=max_mismatch_percent_for_adapter_trimming,
                       three_prime_trim_after_adapter_clip=three_prime_trim_after_adapter_clip,
                       output_type=output_type, sort_bam=sort_bam,
                       max_memory_per_thread_for_bam_sorting=max_memory_per_thread_for_bam_sorting,
                       include_unmapped_reads_in_bam=include_unmapped_reads_in_bam,
                       output_unmapped_reads=output_unmapped_reads, output_dir=alignment_sample_dir,
                       two_pass_mode=two_pass_mode, max_intron_length=max_intron_length)

    def align_miRNA(self, genome_dir, se_read_list, annotation_gtf=None, output_dir="./",
                    max_memory_for_bam_sorting=8000000000, max_alignments_per_read=10, no_soft_clip=True,
                    max_number_of_mismatches=1, max_relative_number_of_mismatches=None,
                    feature_name=None):
        """
        Aligns miRNA according to ENCODE pipeline
        """

        options = " --runThreadN %i" % self.threads
        options += " --genomeDir %s" % os.path.abspath(genome_dir)
        options += " --sjdbGTFfile %s" % annotation_gtf if annotation_gtf else ""

        options += " --alignEndsType EndToEnd" if no_soft_clip else ""   # End-to-end alignment, no soft-clip by default
        if max_relative_number_of_mismatches:
            options += " --outFilterMismatchNoverLmax %f" % max_relative_number_of_mismatches
        else:
            options += " --outFilterMismatchNmax %i" % max_number_of_mismatches  # Max 1 mismatch by default. maximum number of mismatches per pair, large number switches off this filter 7max number of mismatches per pair relative to read length: for 2x100b, max number of mismatches is 0.06*200=8 for the paired read
        options += " --outFilterMultimapScoreRange 0" #the score range below the maximum score for multimapping alignments
        options += " --quantMode TranscriptomeSAM GeneCounts" if annotation_gtf else ""
        options += " --outReadsUnmapped Fastx" # output of unmapped and partially mapped (i.e. mapped only one mateof a paired end read) reads in separate file(s).
        options += " --outSAMtype BAM SortedByCoordinate"
        options += " --outFilterMultimapNmax %i" % max_alignments_per_read # up to 10 alignments by default
        options += " --outSAMunmapped Within"
        options += " --outFilterScoreMinOverLread 0" # alignment will be output only if its score is higher than or equal to this valuenormalized to read length
        options += " --outFilterMatchNminOverLread 0"
        options += " --outFilterMatchNmin 16" # Output alignment if more than <value> bases were aligned
        options += " --alignSJDBoverhangMin 1000" if annotation_gtf else ""
        options += " --alignIntronMax 1" # Max intron length
        options += " --outWigType wiggle"
        options += " --outWigStrand Stranded"
        options += " --outWigNorm RPM"
        options += " --sjdbGTFfeatureExon %s" % feature_name if feature_name else ""
        options += " --outFileNamePrefix %s" % output_dir if output_dir else ""
        options += " --limitBAMsortRAM %i" % max_memory_for_bam_sorting if max_memory_for_bam_sorting else ""
        options += " --readFilesIn %s" % (os.path.abspath(se_read_list) if isinstance(se_read_list, str) else " ".join(map(os.path.abspath, se_read_list)))

        self.execute(options)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
from RouToolPa.Tools.Abstract import Tool


class Glistmaker(Tool):
    """ """
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "glistmaker", path=path, max_threads=max_threads)

    def parse_options(self, in_file_list, output_file, kmer_length=16, frequency_cutoff=None, threads=None,
                      max_tmp_table_number=None, max_tmp_table_size=None):

        options = " %s" % (in_file_list if isinstance(in_file_list, str) else " ".join(in_file_list))
        options += " -o %s" % output_file if output_file else ""
        options += " -w %i" % kmer_length
        options += " -c %i" % frequency_cutoff if frequency_cutoff else ""
        options += " --num_threads %i" % (threads if threads else self.threads)
        options += " --max_tables %i" % max_tmp_table_number if max_tmp_table_number else ""
        options += " --table_size %i" % max_tmp_table_size if max_tmp_table_size else ""

        return options

    def count(self, in_file_list, output_file, kmer_length=16, frequency_cutoff=None, threads=None,
              max_tmp_table_number=None, max_tmp_table_size=None):

        options = self.parse_options(in_file_list, output_file, kmer_length=kmer_length,
                                     frequency_cutoff=frequency_cutoff, threads=threads,
                                     max_tmp_table_number=max_tmp_table_number,
                                     max_tmp_table_size=max_tmp_table_size)

        self.execute(options)

    def generate_kmer_lists_for_primer3(self, in_file_list, species_prefix, threads=None,
                                        max_tmp_table_number=None, max_tmp_table_size=None):
        #out_kmer_16 = "%s_16.list" % species_prefix
        #out_kmer_11 = "%s_11.list" % species_prefix

        options_kmer_16 = self.parse_options(in_file_list, species_prefix, kmer_length=16, threads=threads,
                                             max_tmp_table_number=max_tmp_table_number,
                                             max_tmp_table_size=max_tmp_table_size)
        options_kmer_11 = self.parse_options(in_file_list, species_prefix, kmer_length=11, threads=threads,
                                             max_tmp_table_number=max_tmp_table_number,
                                             max_tmp_table_size=max_tmp_table_size)

        self.execute(options_kmer_16)
        self.execute(options_kmer_11)



__author__ = 'mahajrod'
import os
import pyparsing as pyp
from RouToolPa.Tools.Abstract import JavaTool



class Trimmomatic(JavaTool):

    def __init__(self, java_path="", max_threads=4, jar_path="", jar=None):
        jar = "trimmomatic.jar" if jar is None else jar
        JavaTool.__init__(self, jar, java_path=java_path, max_threads=max_threads,
                          jar_path=jar_path, max_memory=None, timelog="trimmomatic.time.log")

    def parse_options(self, left_reads, output_prefix, output_prefix_left=None, output_extension="fq", right_reads=None,
                      adapters_file=None,
                      mismatch_number=2, pe_reads_score=30, se_read_score=10, min_adapter_len=1,
                      sliding_window_size=None, average_quality_threshold=15,
                      leading_base_quality_threshold=None, trailing_base_quality_threshold=None,
                      crop_length=None, head_crop_length=None, min_length=50, base_quality="phred33"):

        options = " PE" if right_reads else " SE"
        options += " -threads %i" % self.threads
        options += " -%s" % base_quality
        options += " %s" % left_reads
        options += " %s" % right_reads if right_reads else ""
        if right_reads:
            if output_prefix_left:
                options += " %s.pe.%s" % (output_prefix, output_extension)
                options += " %s.se.%s" % (output_prefix, output_extension)
                options += " %s.pe.%s" % (output_prefix_left, output_extension)
                options += " %s.se.%s" % (output_prefix_left, output_extension)
            else:
                options += " %s_1.pe.%s" % (output_prefix, output_extension)
                options += " %s_1.se.%s" % (output_prefix, output_extension)
                options += " %s_2.pe.%s" % (output_prefix, output_extension)
                options += " %s_2.se.%s" % (output_prefix, output_extension)
        else:
            options += " %s.se.%s" % (output_prefix, output_extension)

        options += " ILLUMINACLIP:%s:%i:%i:%i:%i" % (adapters_file, mismatch_number, pe_reads_score,
                                                     se_read_score, min_adapter_len) if adapters_file else ""

        options += " SLIDINGWINDOW:%s:%s" % (sliding_window_size, average_quality_threshold) \
            if sliding_window_size else ""

        options += " LEADING:%i" % leading_base_quality_threshold if leading_base_quality_threshold else ""
        options += " TRAILING:%i" % trailing_base_quality_threshold if trailing_base_quality_threshold else ""

        options += " CROP:%i" % crop_length if crop_length else ""
        options += " HEADCROP:%i" % head_crop_length if head_crop_length else ""

        options += " MINLEN:%i" % min_length if min_length else ""

        return options

    def filter(self, left_reads, output_prefix, output_prefix_left=None, output_extension="fq", right_reads=None, adapters_file=None,
               mismatch_number=2, pe_reads_score=30, se_read_score=10, min_adapter_len=1,
               sliding_window_size=None, average_quality_threshold=15,
               leading_base_quality_threshold=None, trailing_base_quality_threshold=None,
               crop_length=None, head_crop_length=None, min_length=50, logfile="trimmomatic.log",
               base_quality="phred33"):

        options = self.parse_options(left_reads, output_prefix, output_prefix_left=output_prefix_left,
                                     output_extension=output_extension,
                                     right_reads=right_reads, adapters_file=adapters_file,
                                     mismatch_number=mismatch_number, pe_reads_score=pe_reads_score,
                                     se_read_score=se_read_score, min_adapter_len=min_adapter_len,
                                     sliding_window_size=sliding_window_size, average_quality_threshold=average_quality_threshold,
                                     leading_base_quality_threshold=leading_base_quality_threshold,
                                     trailing_base_quality_threshold=trailing_base_quality_threshold,
                                     crop_length=crop_length, head_crop_length=head_crop_length, min_length=min_length,
                                     base_quality=base_quality)

        options += " > %s 2>&1" % logfile if logfile else ""

        self.execute(options=options)

    @staticmethod
    def parse_log(log_file):

        input = pyp.Literal("Input Read Pairs: ")
        input_number = pyp.Word(pyp.nums)
        both = pyp.Literal("Both Surviving: ")
        both_number = pyp.Word(pyp.nums)

        str1 = pyp.Literal("(")
        both_percent = pyp.Word(pyp.nums + ".-")
        forward = pyp.Literal("%) Forward Only Surviving: ")
        forward_number = pyp.Word(pyp.nums)
        forward_percent = pyp.Word(pyp.nums + ".-")
        reverse = pyp.Literal("%) Reverse Only Surviving: ")
        reverse_number = pyp.Word(pyp.nums)
        reverse_percent = pyp.Word(pyp.nums + ".-")
        dropped = pyp.Literal("%) Dropped: ")
        dropped_number = pyp.Word(pyp.nums)
        dropped_percent = pyp.Word(pyp.nums + ".-")
        end = pyp.Literal("%)")

        for token in both_number, forward_number, reverse_number, dropped_number, input_number:
            token.setParseAction(lambda s, l, t: [int(t[0])])
        for token in both_percent, forward_percent, reverse_percent, dropped_percent:
            token.setParseAction(lambda s, l, t: [float(t[0])])

        both_number.setParseAction(lambda s, l, t: [int(t[0])])

        pattern = input + input_number + both + both_number + str1 + both_percent + forward + forward_number + str1 + \
                  forward_percent + reverse + reverse_number + str1 + reverse_percent + dropped + dropped_number + \
                  str1 + dropped_percent + end

        with open(log_file, "r") as log_fd:
            for line in log_fd:
                if line[:17] == "Input Read Pairs:":
                    parsed_list = pattern.parseString(line.strip())

        return {
                "input":                     parsed_list[1],
                "both_surviving":            parsed_list[3],
                "both_surviving,%":          parsed_list[5],
                "forward_only_surviving":    parsed_list[7],
                "forward_only_surviving,%":  parsed_list[9],
                "reverse_only_surviving":    parsed_list[11],
                "reverse_only_surviving,%":  parsed_list[13],
                "dropped":                   parsed_list[15],
                "dropped,%":                 parsed_list[17]
                }






#!/usr/bin/env python

from RouToolPa.Routines.Alignment import AlignmentRoutines


class PacBioRoutines(AlignmentRoutines):
    def __init__(self):
        AlignmentRoutines.__init__(self)

    def extract_not_collapsed_subreads_from_bam(self, subread_bam, ccs_bam, output_prefix):
        """
        ccs_read_names_file = "%s.ccs.reads.ids" % output_prefix
        ccs_read_names_prefix_file = "%s.ccs.reads.prefix.ids" % output_prefix

        SamtoolsV1.get_read_names(ccs_bam, ccs_read_names_file)

        sed_string = "sed \'s/ccs$//\' %s > %s " % (ccs_read_names_file, ccs_read_names_prefix_file)
        os.system(sed_string)
        read_name_list = IdList(filename=ccs_read_names_prefix_file)

        SamtoolsV1.get_reads_by_name(read_name_list, input_sam_fd, output_sam_fd, mode="include", search_mode="exact"):

        """





#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'

import os
from RouToolPa.Tools.Abstract import Tool


class Cookiecutter(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "cookiecutter", path=path, max_threads=max_threads)

    def parse_common_options(self, adapter_file, left_reads, right_reads=None, out_dir="./"):

        options = " --fragments %s" % adapter_file
        #options += " -t %i" % self.threads
        options += " -o %s" % out_dir
        options += " -i %s" % left_reads if right_reads is None else ""     # single-end data
        options += " -1 %s -2 %s" % (left_reads, right_reads) if right_reads else ""    # paired-end data

        return options

    def remove(self, adapter_file, left_reads, right_reads=None, out_dir="./"):
        options = self.parse_common_options(adapter_file, left_reads, right_reads=right_reads, out_dir=out_dir)
        self.execute(options, cmd="remove")

    def extract(self, adapter_file, left_reads, right_reads=None, out_dir="./"):
        options = self.parse_common_options(adapter_file, left_reads, right_reads=right_reads, out_dir=out_dir)
        self.execute(options, cmd="extract")

    def separate(self, adapter_file, left_reads, right_reads=None, out_dir="./"):
        options = self.parse_common_options(adapter_file, left_reads, right_reads=right_reads, out_dir=out_dir)
        self.execute(options, cmd="separate")

    def rm_reads(self, adapter_file, left_reads, stats_file, right_reads=None, out_dir="./", use_dust_filter=False,
                 dust_cutoff=None, dust_window_size=None, use_N_filter=False,
                 read_length_cutoff=None, polyGC_length_cutoff=None):
        options = self.parse_common_options(adapter_file, left_reads, right_reads=right_reads, out_dir=out_dir)
        options += " -p %i" % polyGC_length_cutoff if polyGC_length_cutoff else ""
        options += " -l %i" % read_length_cutoff if read_length_cutoff else ""
        options += " -d" if use_dust_filter else ""
        options += " -c %i" % dust_cutoff if dust_cutoff else ""
        options += " -k %i" % dust_window_size if dust_window_size else ""
        options += " -N" if use_N_filter else ""
        options += " > %s" % stats_file

        self.execute(options, cmd="rm_reads")


class CookiecutterOld(Tool):
    def __init__(self, path="", max_threads=1):
        Tool.__init__(self, "cookiecutter", path=path, max_threads=max_threads)

    def rm_reads(self, adapter_file, left_reads, stats_file, right_reads=None, out_dir="./"):
        options = " --fragments %s" % adapter_file
        options += " -o %s" % out_dir
        options += " -i %s" % left_reads if right_reads is None else ""     # single-end data
        options += " -1 %s -2 %s" % (left_reads, right_reads) if right_reads else ""    # paired-end data
        options += " > %s" % stats_file

        self.execute(options, cmd="rm_reads")

if __name__ == "__main__":
    pass





#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import JavaTool


class BaseRecalibrator(JavaTool):

    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T BaseRecalibrator", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def get_recalibration_table(self, reference, alignment, output_table, known_sites_vcf, BQSR=None,
                                include_region_id_file=None, exclude_region_id_file=None):

        # TODO: add rest of  options
        options = ""
        options += " -R %s" % reference
        options += " -I %s" % alignment
        options += " -nct %i" % self.threads
        options += " -BQSR %s" % BQSR if BQSR else ""
        options += " -knownSites %s" % known_sites_vcf if isinstance(known_sites_vcf, str) else " -knownSites ".join(known_sites_vcf)
        options += " -L %s" % include_region_id_file if include_region_id_file else ""
        options += " -XL %s" % exclude_region_id_file if exclude_region_id_file else ""
        options += " -o %s" % output_table

        self.execute(options)

if __name__ == "__main__":
    import os
    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc"
    os.chdir(workdir)
    #gatk_dir =
    reference = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/reference/Alsu24mc/Alsu24mc.fasta"
    alignment = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc/001_trimmed_sorted_rm_pcr_chrom_corrected.bam"
    corrected_alignment = "001_trimmed_sorted_rm_pcr_chrom_corrected.bam"
    BaseRecalibrator = BaseRecalibrator(jar_path="/home/mahajrod/Repositories/genetic/NGS_tools/GenomeAnalysisTK-3.2-0")
    BaseRecalibrator.get_recalibration_table(reference, alignment)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
from RouToolPa.Tools.Abstract import Tool
from RouToolPa.Collections.General import SynDict, TwoLvlDict



class HTSeq(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "htseq-count", path=path, max_threads=max_threads)

    def count(self, alignment_file, gff_file, output_file, samtype="bam", order=None, stranded_rnaseq="yes",
              min_alignment_quality=10, feature_type="exon", feature_id_attribute="gene_id", mode="union",
              suppress_progres_report=False):
        """
          -h, --help            show this help message and exit
          -f SAMTYPE, --format=SAMTYPE
                                type of <alignment_file> data, either 'sam' or 'bam'
                                (default: sam)
          -r ORDER, --order=ORDER
                                'pos' or 'name'. Sorting order of <alignment_file>
                                (default: name). Paired-end sequencing data must be
                                sorted either by position or by read name, and the
                                sorting order must be specified. Ignored for single-
                                end data.
          -s STRANDED, --stranded=STRANDED
                                whether the data is from a strand-specific assay.
                                Specify 'yes', 'no', or 'reverse' (default: yes).
                                'reverse' means 'yes' with reversed strand
                                interpretation
          -a MINAQUAL, --minaqual=MINAQUAL
                                skip all reads with alignment quality lower than the
                                given minimum value (default: 10)
          -t FEATURETYPE, --type=FEATURETYPE
                                feature type (3rd column in GFF file) to be used, all
                                features of other type are ignored (default, suitable
                                for Ensembl GTF files: exon)
          -i IDATTR, --idattr=IDATTR
                                GFF attribute to be used as feature ID (default,
                                suitable for Ensembl GTF files: gene_id)
          -m MODE, --mode=MODE  mode to handle reads overlapping more than one feature
                                (choices: union, intersection-strict, intersection-
                                nonempty; default: union)
          -o SAMOUT, --samout=SAMOUT
                                write out all SAM alignment records into an output SAM
                                file called SAMOUT, annotating each line with its
                                feature assignment (as an optional field with tag
                                'XF')
          -q, --quiet           suppress progress report
        """

        options = " -f %s" % samtype
        options += " -r %s" % order if order else ""
        options += " -s %s" % stranded_rnaseq if stranded_rnaseq else ""
        options += " -a %i" % min_alignment_quality if min_alignment_quality else ""
        options += " -t %s" % feature_type if feature_type else ""
        options += " -i %s" % feature_id_attribute if feature_id_attribute else ""
        options += " -m %s" % mode if mode else ""
        options += " -q" if suppress_progres_report else ""

        options += " %s" % alignment_file
        options += " %s" % gff_file
        options += " > %s" % output_file

        self.execute(options=options)

    @staticmethod
    def combine_count_files(count_file_list, output_file, sample_name_list=None):

        if sample_name_list is not None:
            if len(count_file_list) != len(sample_name_list):
                raise ValueError("Several files doesn't have corresponding sample name")

        samples = zip(sample_name_list if sample_name_list else count_file_list, count_file_list)

        count_table = TwoLvlDict()

        for sample, filename in samples:
            count_table[sample] = SynDict(filename=filename, header=False, separator="\t", allow_repeats_of_key=False,
                                          split_values=False, values_separator=",", key_index=0, value_index=1,
                                          close_after_if_file_object=False, expression=None, comments_prefix="__")

        count_table.write(output_file)



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
from RouToolPa.Tools.Picard.MarkDuplicates import MarkDuplicates
from RouToolPa.Tools.Picard.SortVcf import SortVcf
from RouToolPa.Tools.Picard.AddOrReplaceReadGroups import AddOrReplaceReadGroups
from RouToolPa.Tools.Picard.CreateSequenceDictionary import CreateSequenceDictionary


SortVcf = SortVcf()
MarkDuplicates = MarkDuplicates()
AddOrReplaceReadGroups = AddOrReplaceReadGroups()
CreateSequenceDictionary = CreateSequenceDictionary()



#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import JavaTool



class VariantRecalibrator(JavaTool):
    # TODO: fix problems with unrecognized data
    # http://www.broadinstitute.org/gatk/gatkdocs/org_broadinstitute_gatk_tools_walkers_variantrecalibration_VariantRecalibrator.html

    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T VariantRecalibrator", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    def build_model(self, reference_file, input_vcf, training_set_good,
                       training_set_bad=None, mode="SNP",
                       recalfile="output_recal",
                       tranchesfile="output.tranches",
                       #rscriptfile="output.plots.R",
                       GATK_dir=""):
        # possible mode:
        # SNP
        # INDEL
        bad_set = ""
        if training_set_bad:
            bad_set = "-resource:bad,known=false,training=true,bad=true %s" % training_set_bad
            """
            print("java -Xmx4g -jar %sGenomeAnalysisTK.jar -T VariantRecalibrator -R %s -input %s -an QD \
                  -an HaplotypeScore -an MQRankSum -an ReadPosRankSum -an FS -an MQ \
                  -mode %s -resource:good,training=true,truth=true %s %s -recalFile %s -tranchesFile %s -rscriptFile %s"
                  % (GATK_dir, reference_file, input_vcf, mode, training_set_good, bad_set, recalfile, tranchesfile, rscriptfile))
            """
        os.system("java -Xmx4g -jar %sGenomeAnalysisTK.jar -T VariantRecalibrator -R %s -input %s -an QD \
                  -an HaplotypeScore -an MQRankSum -an ReadPosRankSum -an FS -an MQ \
                  -mode %s -resource:good,known=true,training=true,truth=true %s %s -recalFile %s -tranchesFile %s"
                  % (GATK_dir, reference_file, input_vcf, mode, training_set_good, bad_set, recalfile, tranchesfile))




#!/usr/bin/env python
"""
Last output parser Module based on pandas
"""
__author__ = 'Sergei F. Kliver'
import datetime
from copy import deepcopy
from collections import OrderedDict

import numpy as np
import pandas as pd

import RouToolPa.Formats.AlignmentFormats as AlignmentFormats


class CollectionLinks:

    def __init__(self, in_file=None, records=None, format="tab", parsing_mode="complete",
                 target_black_list=(), target_white_list=(),
                 query_black_list=(), query_white_list=(),
                 target_syn_dict=None, query_syn_dict=None,
                 min_target_hit_len=None, min_query_hit_len=None,
                 min_target_len=None, min_query_len=None):

        self.formats = ["tab", "busco_tblastn"]
        self.parsing_parameters = {"tab": {"complete": {
                                                   "col_names": ["hic_scaffold_id",
                                                                 "raw_scaffold_id",
                                                                 "raw_scaffold_start",
                                                                 "raw_scaffold_end",
                                                                 "raw_scaffold_strand",
                                                                 "hic_scaffold_start",
                                                                 "hic_scaffold_end"],
                                                   "cols":      [0, 1, 2, 3, 4, 5, 6],
                                                   "index_cols": ["hic_scaffold_id"],
                                                   "converters": {
                                                                  "hic_scaffold_id":        str,
                                                                  "raw_scaffold_id":        str,
                                                                  "raw_scaffold_start":     np.int64,
                                                                  "raw_scaffold_end":       np.int64,
                                                                  "raw_scaffold_strand":    str,
                                                                  "hic_scaffold_start":     np.int64,
                                                                  "hic_scaffold_end":       np.int64
                                                                 },
                                                   "col_name_indexes": {
                                                                        "hic_scaffold_id":      0,
                                                                        "raw_scaffold_id":      1,
                                                                        "raw_scaffold_start":   2,
                                                                        "raw_scaffold_end":     3,
                                                                        "raw_scaffold_strand":  4,
                                                                        "hic_scaffold_start":   5,
                                                                        "hic_scaffold_end":     6
                                                                        },
                                                   "original_col_names": ["hic_scaffold_id",
                                                                          "raw_scaffold_id",
                                                                          "raw_scaffold_start",
                                                                          "raw_scaffold_end",
                                                                          "raw_scaffold_strand",
                                                                          "hic_scaffold_start",
                                                                          "hic_scaffold_end"],
                                                   "original_col_name_indexes": {
                                                                                 "hic_scaffold_id":      0,
                                                                                 "raw_scaffold_id":      1,
                                                                                 "raw_scaffold_start":   2,
                                                                                 "raw_scaffold_end":     3,
                                                                                 "raw_scaffold_strand":  4,
                                                                                 "hic_scaffold_start":   5,
                                                                                 "hic_scaffold_end":     6
                                                                                 },
                                                   },

                                           },

                                    }

        self.parsing_mode = parsing_mode

        self.format = format
        self.target_black_list = target_black_list
        self.target_white_list = target_white_list
        self.target_syn_dict = target_syn_dict

        self.query_black_list = query_black_list
        self.query_white_list = query_white_list
        self.query_syn_dict = query_syn_dict
        # attributes type conversion parameters

        self.converters = OrderedDict()
        self.pandas_int_type_correspondence = OrderedDict({
                                                           "Int8":  np.float16,
                                                           "Int16": np.float16,
                                                           "Int32": np.float32,
                                                           "Int64": np.float64,
                                                           })

        # init aliases
        self.col_names = self.parsing_parameters[self.format][self.parsing_mode]["col_names"]
        self.index_cols = self.parsing_parameters[self.format][self.parsing_mode]["index_cols"]
        #self.target_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_id"]
        #self.target_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_start"]
        #self.target_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_hit_len"]
        #self.target_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["target_strand"]

        #self.query_id_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_id"]
        #self.query_start_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_start"]
        #self.query_hit_len_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_hit_len"]
        #self.query_strand_index = self.parsing_parameters[self.format][self.parsing_mode]["col_name_indexes"]["query_strand"]

        #
        self.target_scaffold_list = None
        self.query_scaffold_list = None
        self.target_scaffold_lengths = None
        self.query_scaffold_lengths = None

        if in_file:
            self.read(in_file,
                      format=format,
                      parsing_mode=parsing_mode,
                      target_black_list=target_black_list,
                      target_white_list=target_white_list,
                      query_black_list=query_black_list,
                      query_white_list=query_white_list,
                      min_target_hit_len=min_target_hit_len,
                      min_query_hit_len=min_query_hit_len,
                      min_target_len=min_target_len,
                      min_query_len=min_query_len)

        else:
            self.records = records

    def read(self, in_file,
             format="tab", parsing_mode="all",
             target_black_list=(), target_white_list=(),
             query_black_list=(), query_white_list=(),
             min_target_hit_len=None, min_query_hit_len=None,
             min_target_len=None, min_query_len=None):
        if format not in self.parsing_parameters:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing!" % parsing_mode)
        elif parsing_mode not in self.parsing_parameters[format]:
            raise ValueError("ERROR!!! This format(%s) was not implemented yet for parsing in this mode(%s)!" % (format, parsing_mode))

        print("%s\tReading input..." % str(datetime.datetime.now()))
        self.records = pd.read_csv(in_file, sep='\t', header=None, na_values=".",
                                   comment="#",
                                   usecols=self.parsing_parameters[format][parsing_mode]["cols"],
                                   converters=self.parsing_parameters[format][parsing_mode]["converters"],
                                   names=self.parsing_parameters[format][parsing_mode]["col_names"],
                                   index_col=self.parsing_parameters[format][parsing_mode]["index_cols"])

        print("%s\tReading input finished..." % str(datetime.datetime.now()))

    @staticmethod
    def get_filtered_entry_list(entry_list,
                                entry_black_list=[],
                                sort_entries=False,
                                entry_ordered_list=None,
                                entry_white_list=[]):
        white_set = set(entry_white_list)
        black_set = set(entry_black_list)
        entry_set = set(entry_list)

        if white_set:
            entry_set = entry_set & white_set
        if black_set:
            entry_set = entry_set - black_set

        filtered_entry_list = list(entry_set)
        if sort_entries:
            filtered_entry_list.sort()

        final_entry_list = []

        if entry_ordered_list:
            for entry in entry_ordered_list:
                if entry in filtered_entry_list:
                    final_entry_list.append(entry)
                    filtered_entry_list.remove(entry)
                else:
                    print("WARNING!!!Entry(%s) from order list is absent in list of entries!" % entry)
            return final_entry_list + filtered_entry_list
        else:
            return filtered_entry_list

    def write(self, output, separator="\t", header=False):
        self.records.to_csv(output,
                            sep=separator,
                            header=header,
                            index=False)

    def get_raw_coordinate(self, hic_scaffold, start, end):
        hic_scaffold_df = self.records.loc[hic_scaffold]
        hic_scaffold_df = hic_scaffold_df[(hic_scaffold_df["hic_scaffold_start"] <= start) & (hic_scaffold_df["hic_scaffold_end"] >= end)]

        if len(hic_scaffold_df["raw_scaffold_id"]) > 1:
            print "Ambigious coordinates"
            return None
        elif hic_scaffold_df.empty:
            return None

        if hic_scaffold_df["raw_scaffold_strand"][0] == "+":
            raw_start = start - hic_scaffold_df["hic_scaffold_start"][0]
            raw_end = end - hic_scaffold_df["hic_scaffold_start"][0]
        else:
            raw_start = hic_scaffold_df["raw_scaffold_end"][0] - start + hic_scaffold_df["hic_scaffold_start"][0]
            raw_end = hic_scaffold_df["raw_scaffold_end"][0] - end + hic_scaffold_df["hic_scaffold_start"][0]

        return hic_scaffold_df["raw_scaffold_id"][0], raw_start, raw_end









#!/usr/bin/env python

__author__ = 'mahajrod'

from RouToolPa.Tools.Abstract import JavaTool


class IndelRealigner(JavaTool):
    def __init__(self,  java_path="", max_threads=4, jar_path="", max_memory=None, timelog=None):
        JavaTool.__init__(self, "GenomeAnalysisTK.jar -T IndelRealigner", java_path=java_path,
                          max_threads=max_threads, jar_path=jar_path, max_memory=max_memory,
                          timelog=timelog)

    # "find" method was renamed to realign
    def realign(self, reference, alignment, corrected_alignment, target_intervals="forIndelRealigner.intervals",
                known_indels_vcf=None, model=None, lod_threshold=None, entropy_threshold=None, max_cons=None,
                max_size_for_movement=None, max_pos_move=None, max_reads_for_cons=None,
                max_reads_for_realignment=None, max_reads_in_memory=None, no_original_tags=False,
                nway_out=False, default_base_qualities=None):

        options = "" #" -nt %i" % self.threads
        options += " -R %s" % reference
        options += " -I %s" % alignment
        options += " -o %s" % corrected_alignment
        options += " -targetIntervals %s" % target_intervals
        options += " --known %s" % known_indels_vcf if known_indels_vcf else ""
        options += " -model %s" % model if model else ""
        options += " -LOD %i" % lod_threshold if lod_threshold else ""
        options += " -entropy %i" % entropy_threshold if entropy_threshold else ""
        options += " --maxConsensuses %i" % max_cons if max_cons else ""
        options += " -maxisize %i" % max_size_for_movement if max_size_for_movement else ""
        options += " -maxPosMove %i" % max_pos_move if max_pos_move else ""
        options += " -greedy %i" % max_reads_for_cons if max_reads_for_cons else ""
        options += " -maxReads %i" % max_reads_for_realignment if max_reads_for_realignment else ""
        options += " -maxinMemory %i" % max_reads_in_memory if max_reads_in_memory else ""

        options += " --nWayOut" if nway_out else ""
        options += " -noTags" if no_original_tags else ""
        options += " --defaultBaseQualities %i" % default_base_qualities if default_base_qualities else ""

        self.execute(options)

if __name__ == "__main__":
    import os
    workdir = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc"
    os.chdir(workdir)
    #gatk_dir =
    reference = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/reference/Alsu24mc/Alsu24mc.fasta"
    alignment = "/media/mahajrod/d9e6e5ee-1bf7-4dba-934e-3f898d9611c8/Data/Alsu/fastq/001/alignment_tmap_Alsu24mc/001_trimmed_sorted_rm_pcr_chrom.bam"
    corrected_alignment = "001_trimmed_sorted_rm_pcr_chrom_corrected.bam"
    IndelRealigner = IndelRealigner(jar_path="/home/mahajrod/Repositories/genetic/NGS_tools/GenomeAnalysisTK-3.2-0")
    IndelRealigner.find(reference, alignment, corrected_alignment)



#!/usr/bin/env python

from collections import OrderedDict


from RouToolPa.Collections.General import TwoLvlDict

class RecordBARRNAP():
    def __init__(self, start, end, strand, type, length, expected_length, partial):

        self.start = start                                          # int
        self.end = end                                              # int
        self.strand = strand                                        # str
        self.type = type                                            # str
        self.length = length                                        # int
        self.expected_length = expected_length                      # int
        self.partial = partial                                      # boolean
        self.length_ratio = float(length) / float(expected_length) if expected_length else None

    def attributes_string(self):
        attributes_string = "Len=%i;Exp_len=%i;Partial=%s" % (self.length, self.expected_length, str(self.partial))

        attributes_string += (";Ratio=%f" % self.length_ratio) if self.length_ratio else ""

        return attributes_string

    def gff_str(self):
        #source	type	start	end	score	strand	phase	attributes
        return "Barrnap\t%s\t%i\t%i\t.\t%s\t.\t%s" % (self.type, self.start, self.end, self.strand,
                                                      self.attributes_string())

"""
    def simple_gff_str(self):
        # seqid	source	type	start	end	score	strand	phase	attributes
        attributes_string = "Period=%i;N_copies=%.1f;Pattern=%s" % (self.period, self.number_of_copies, self.pattern)

        return "TRF\trepeat\t%i\t%i\t.\t.\t.\t%s" % (self.start, self.end, attributes_string)

    def table_str_short(self):

        return "%i\t%i\t%i\t%.1f\t%s\t%s" % (self.start, self.end, self.period, self.number_of_copies,
                                             self.pattern, self.tandem_repeat)

    def table_str_long(self):

        return "%i\t%i\t%i\t%.1f\t%i\t%i\t%i\t%i\t%s\t%.2f\t%s\t%s" % (self.start, self.end, self.period,
                                                                       self.number_of_copies,
                                                                       self.consensus_pattern_size,
                                                                       self.percent_of_matches,
                                                                       self.percent_of_indels,
                                                                       self.alignment_score,
                                                                       ",".join(map(lambda x: str(x), self.nucleotide_percent_list)),
                                                                       self.entropy,
                                                                       self.pattern, self.tandem_repeat)

"""


class CollectionBARRNAP():

    def __init__(self, record_dict=None, e_value_cutoff=None, partial_threshold=None, reject_threshold=None,
                 barrnap_file=None, from_file=True):
        if from_file:
            self.records = OrderedDict()
            with open(barrnap_file, "r") as fd:
                chrom = None
                for line in fd:
                    if "Setting evalue cutoff" in line:
                        self.e_value_cutoff = line.strip().split()[-1]
                    elif "Will tag genes" in line:
                        self.partial_threshold = float(line.strip().split()[-4])
                    elif "Will reject genes" in line:
                        self.reject_threshold = float(line.strip().split()[-4])
                    if "Found:" in line:
                        self._add_record(line)
        else:
            self.records = record_dict
            self.e_value_cutoff = e_value_cutoff            # e-value cuttof for hit
            self.partial_threshold = partial_threshold      # length threshold to mark gene as partial (if below)
            self.reject_threshold = reject_threshold        # length threshold to reject gene (if below)

    def _add_record(self, line):
        line_list = line.strip().split("Found: ")[-1].split()
        type = line_list[0]
        chrom = line_list[1]
        length, expected_length = map(int, line_list[2].split("=")[-1].split("/"))
        start, end = map(int, line_list[3].split(".."))
        strand = line_list[4]
        partial = "partial" in line_list[-1]
        record = RecordBARRNAP(start, end, strand, type, length, expected_length, partial)
        if chrom not in self.records:
            self.records[chrom] = []
        self.records[chrom].append(record)

    def get_annotated_types(self):

        annotated_types = set()
        for chrom in self.records:
            for record in self.records[chrom]:
                annotated_types.add(record.type)

        return annotated_types

    def count_types(self, output_file=None, total_output_file=None, return_mode="chrom"):

        annotated_types = self.get_annotated_types()
        count_dict = TwoLvlDict()
        total_count_dict = OrderedDict()

        for type in annotated_types:
            total_count_dict[type] = OrderedDict()
            total_count_dict[type]["complete"] = 0
            total_count_dict[type]["partial"] = 0

        for chrom in self.records:
            count_dict[chrom] = OrderedDict()
            for type in annotated_types:
                count_dict[chrom][type] = 0

        for chrom in self.records:
            for record in self.records[chrom]:
                count_dict[chrom][record.type] += 1
                if record.partial:
                    total_count_dict[record.type]["partial"] += 1
                else:
                    total_count_dict[record.type]["complete"] += 1

        if output_file:
            count_dict.write(output_file)

        if total_output_file:
            with open(total_output_file, "w") as out_fd:
                out_fd.write("#rRNA\tComplete%s\tPartial%s\n" % ("(>%.2f of expected length)" % self.partial_threshold if self.partial_threshold else "",
                                                                 "(<%.2f of expected length)" % self.partial_threshold if self.partial_threshold else ""))
                for type in total_count_dict:
                    out_fd.write("%s\t%i\t%i\n" % (type, total_count_dict[type]["complete"],
                                                   total_count_dict[type]["partial"]))

        if return_mode == "chrom":
            return count_dict
        elif return_mode == "total":
            return total_count_dict
        elif return_mode == "both":
            return count_dict, total_count_dict
        else:
            raise ValueError("Unknown return type. Allowed variants: 'chrom', 'total', 'both'")

    def write_gff(self, out_file, write_chr_string=False):
        with open(out_file, "w") as out_fd:
            for chrom in self.records:
                if write_chr_string:
                    out_fd.write("#%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.gff_str()))

    def write(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#>chrom\n#\tstart\tend\tperiod\tnumber_of_copies\tpattern\ttandem_repeat\n")
            for chrom in self.records:
                out_fd.write(">%s\n" % chrom)
                for record in self.records[chrom]:
                    out_fd.write("\t%s\n" % record.table_str_short())

    def write_short_table(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#chrom\tstart\tend\tperiod\tnumber_of_copies\tpattern\ttandem_repeat\n")
            for chrom in self.records:
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.table_str_short()))

    def write_wide_table(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write("#chrom\tstart\tend\tperiod\tnumber_of_copies\tconsensus_pattern_size\tpercent_of_matches\tpercent_of_indels\talignment_score\tnucleotide_composition\tentropy\tpattern\ttandem_repeat")
            for chrom in self.records:
                for record in self.records[chrom]:
                    out_fd.write("%s\t%s\n" % (chrom, record.table_str_long()))




#!/usr/bin/env python
import os
from RouToolPa.Tools.Abstract import Tool



class SWAMP(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "SWAMP.py", path=path, max_threads=max_threads)

    @staticmethod
    def parse_common_options(input_folder=None, branch_names_file=None, window_size=None,
                             substitution_threshold=None, min_seq_len=None, interscan_masking=False):

        options = " -i %s" % input_folder if input_folder else ""
        options += " -b %s" % branch_names_file if branch_names_file else ""
        options += " -t %i" % substitution_threshold if substitution_threshold else ""
        options += " -w %i" % window_size if window_size else ""
        options += " -m %i" % min_seq_len if min_seq_len else ""
        options += " -s" if interscan_masking else ""

    def mask_alignments(self, input_folder, output_folder, ):

        self.safe_mkdir(output_folder)
        for filename in os.listdir(input_folder):
            pass


__author__ = 'mahajrod'
import os
from collections import OrderedDict
from Bio.Seq import Seq
from RouToolPa.Collections.General import TwoLvlDict
from RouToolPa.Routines.File import FileRoutines


class FastQRoutines(FileRoutines):

    def __init__(self):
        pass

    def reverse_complement(self, in_file, out_file):
        with self.metaopen(in_file, "r") as in_fd:
            with self.metaopen(out_file, "w") as out_fd:
                for line in in_fd:
                    out_fd.write(line)
                    out_fd.write(str(Seq(in_fd.next().strip()).reverse_complement()))
                    out_fd.write("\n")
                    out_fd.write(in_fd.next())
                    out_fd.write(in_fd.next().strip()[::-1])
                    out_fd.write("\n")

    @staticmethod
    def parse_illumina_name(string):
        name_list = string[1:].split(" ")
        return tuple(name_list[0].split(":"))

    def remove_tiles_from_fastq(self, forward_reads, black_list_forward_tiles_list,
                                reverse_reads, black_list_reverse_tiles_list, output_prefix):

        filtered_paired_forward_pe = "%s.ok.pe_1.fastq" % output_prefix
        filtered_forward_se = "%s.ok.forward.se.fastq" % output_prefix
        filtered_out_forward_se = "%s.bad.forward.fastq" % output_prefix

        filtered_paired_reverse_pe = "%s.ok.pe_2.fastq" % output_prefix
        filtered_reverse_se = "%s.ok.reverse.se.fastq" % output_prefix
        filtered_out_reverse_se = "%s.bad.reverse.fastq" % output_prefix

        forward_input_fd = self.metaopen(forward_reads, "r")
        reverse_input_fd = self.metaopen(reverse_reads, "r")

        filtered_paired_forward_pe_fd = self.metaopen(filtered_paired_forward_pe, "w")
        filtered_forward_se_fd = self.metaopen(filtered_forward_se, "w")
        filtered_out_forward_se_fd = self.metaopen(filtered_out_forward_se, "w")

        filtered_paired_reverse_pe_fd = self.metaopen(filtered_paired_reverse_pe, "w")
        filtered_reverse_se_fd = self.metaopen(filtered_reverse_se, "w")
        filtered_out_reverse_se_fd = self.metaopen(filtered_out_reverse_se, "w")

        for line in forward_input_fd:
            name_list = line.strip()[1:].split(":")
            #instrument_id = name_list[0]
            #run_id = name_list[1]
            #flowcell_id = name_list[2]
            #lane_id = name_list[3]
            tile_id = name_list[4]

            if (tile_id in black_list_forward_tiles_list) and (tile_id in black_list_reverse_tiles_list):
                filtered_out_forward_se_fd.write(line)
                for i in range(0, 3):
                    filtered_out_forward_se_fd.write(forward_input_fd.next())
                for i in range(0, 4):
                    filtered_out_reverse_se_fd.write(reverse_input_fd.next())

            elif (tile_id in black_list_forward_tiles_list) and (not(tile_id in black_list_reverse_tiles_list)):
                filtered_out_forward_se_fd.write(line)
                for i in range(0, 3):
                    filtered_out_forward_se_fd.write(forward_input_fd.next())
                for i in range(0, 4):
                    filtered_reverse_se_fd.write(reverse_input_fd.next())

            elif (not (tile_id in black_list_forward_tiles_list)) and (tile_id in black_list_reverse_tiles_list):
                filtered_forward_se_fd.write(line)
                for i in range(0, 3):
                    filtered_forward_se_fd.write(forward_input_fd.next())
                for i in range(0, 4):
                    filtered_out_reverse_se_fd.write(reverse_input_fd.next())

            else:
                filtered_paired_forward_pe_fd.write(line)
                for i in range(0, 3):
                    filtered_paired_forward_pe_fd.write(forward_input_fd.next())
                for i in range(0, 4):
                    filtered_paired_reverse_pe_fd.write(reverse_input_fd.next())

        filtered_paired_forward_pe_fd.close()
        filtered_forward_se_fd.close()
        filtered_out_forward_se_fd.close()

        filtered_paired_reverse_pe_fd.close()
        filtered_reverse_se_fd.close()
        filtered_out_reverse_se_fd.close()

    def combine_fastq_files(self, samples_directory, sample, output_directory,
                            use_links_if_merge_not_necessary=True, input_is_se=False):
        sample_dir = "%s/%s/" % (samples_directory, sample)

        filetypes, forward_files, reverse_files, se_files = self.make_lists_forward_and_reverse_files(sample_dir,
                                                                                                      input_is_se=input_is_se)

        uncompresed = True
        if len(filetypes) == 1:
            if ("fq.gz" in filetypes) or ("fastq.gz" in filetypes):
                command = "zcat"
                uncompresed = False
            elif ("fq.bz2" in filetypes) or ("fastq.bz2" in filetypes):
                command = "bzcat"
                uncompresed = False
            else:
                command = "cat"

            merged_forward = "%s/%s_1.fq" % (output_directory, sample)
            merged_reverse = "%s/%s_2.fq" % (output_directory, sample)
            merged_se = "%s/%s.se.fq" % (output_directory, sample)

            if use_links_if_merge_not_necessary and (len(forward_files) == 1) and (len(reverse_files) == 1) and (uncompresed == True):
                forward_string = "ln -s %s %s" % (forward_files[0], merged_forward)
                reverse_string = "ln -s %s %s" % (reverse_files[0], merged_reverse)

                print("Executing: %s" % forward_string)
                os.system(forward_string)
                print("Executing: %s" % reverse_string)
                os.system(reverse_string)

                if len(se_files) == 1:
                    os.system("ln -s %s %s" % (se_files[0], merged_se))
                    return merged_forward, merged_reverse, merged_se
                elif len(se_files) > 0:
                    os.system("%s %s > %s" % (command, " ".join(se_files), merged_se))
                    return merged_forward, merged_reverse, merged_se
                else:
                    return merged_forward, merged_reverse, None
            else:
                if (len(forward_files) > 0) and (len(reverse_files) > 0):
                    os.system("%s %s > %s" % (command, " ".join(forward_files), merged_forward))
                    os.system("%s %s > %s" % (command, " ".join(reverse_files), merged_reverse))
                    if len(se_files) > 0:
                        os.system("%s %s > %s" % (command, " ".join(se_files), merged_se))
                        return merged_forward, merged_reverse, merged_se
                    else:
                        return merged_forward, merged_reverse, None
                if len(se_files) > 0:
                    os.system("%s %s > %s" % (command, " ".join(se_files), merged_se))
                    return None, None, merged_se
                else:
                    raise IOError("No input files were found")
        else:
            raise IOError("Extracting from mix of archives in not implemented yet")

    @staticmethod
    def filter_se_by_length(input_file, filtered_file, filtered_out_file, min_len=None, max_len=None):

        if min_len and max_len:
            def expression(line):
                return min_len <= len(line) <= max_len
        elif min_len:
            def expression(line):
                return min_len <= len(line)
        elif max_len:
            def expression(line):
                return len(line) <= max_len
        else:
            raise ValueError("Both minimum and maximum thresholds for read length were not set")

        with open(input_file, "r") as in_fd:
            with open(filtered_file, "w") as filtered_fd:
                with open(filtered_out_file, "w") as filtered_out_fd:
                    for read_name in in_fd:
                        read = in_fd.next()
                        #print len(read.strip())
                        #print expression(read.strip())
                        if expression(read.strip()):
                            filtered_fd.write(read_name)
                            filtered_fd.write(read)
                            filtered_fd.write(in_fd.next())
                            filtered_fd.write(in_fd.next())
                        else:
                            filtered_out_fd.write(read_name)
                            filtered_out_fd.write(read)
                            filtered_out_fd.write(in_fd.next())
                            filtered_out_fd.write(in_fd.next())

    def split_illumina_fastq_by_lanes(self, input_fastq, output_dir, output_prefix=None, output_suffix=".fastq"):
        out_fd_dict = OrderedDict()

        with self.metaopen(input_fastq, "r") as in_fd:
            for line in in_fd:
                tmp = line.split(":")
                lane_id = ".".join(tmp[:4])[1:]

                if lane_id not in out_fd_dict:
                    out_fd_dict[lane_id] = open("%s/%s%s" % (output_dir,
                                                             ((output_prefix + ".") if output_prefix else "") + lane_id,
                                                             output_suffix), "w")

                out_fd_dict[lane_id].write(line)

                for i in range(0, 3):
                    out_fd_dict[lane_id].write(in_fd.next())

        for fd in out_fd_dict:
            out_fd_dict[fd].close()

    def find_tiles(self, fastq_file, output_file):
        with self.metaopen(fastq_file, "r") as fastq_fd, (output_file if isinstance(output_file, file) else open(output_file, "w")) as out_fd:
            out_fd.write("#Machine\tRun\tFlowcellID\tLane\tTile\n")
            tile_set = set()

            for line in fastq_fd:
                read_name_tuple = self.parse_illumina_name(line)[:5]
                tile_set.add(read_name_tuple)
                for i in 0, 1, 2:
                    fastq_fd.next()

            tile_list = list(tile_set)
            tile_list.sort()

            for tile in tile_list:
                out_fd.write("\t".join(tile))
                out_fd.write("\n")

    def count_reads_in_tiles(self, fastq_file, output_file):
        with self.metaopen(fastq_file, "r") as fastq_fd, (output_file if isinstance(output_file, file) else open(output_file, "w")) as out_fd:
            out_fd.write("#Machine\tRun\tFlowcellID\tLane\tTile\tReadNumber\n")
            tile_dict = OrderedDict

            for line in fastq_fd:
                tile_name = "\t".join(self.parse_illumina_name(line)[:5])
                if tile_name in tile_dict:
                    tile_dict[tile_name] += 1
                else:
                    tile_dict[tile_name] = 1

                for i in 0, 1, 2:
                    fastq_fd.next()

            for tile_name in sorted(tile_dict.keys()):
                out_fd.write(tile_name)
                out_fd.write("\t")
                out_fd.write(str(tile_dict[tile_name]))
                out_fd.write("\n")

    def count_reads_and_bases(self, fastq_file_list, stat_file=None):

        fastq_list = [fastq_file_list] if isinstance(fastq_file_list, str) else fastq_file_list

        counts = TwoLvlDict()

        for fastq_file in fastq_list:
            counts[fastq_file] = OrderedDict()
            counts[fastq_file]["Reads"] = 0
            counts[fastq_file]["Bases"] = 0

        for fastq_file in fastq_list:
            with self.metaopen(fastq_file, "r") as fastq_fd:
                for line in fastq_fd:
                    counts[fastq_file]["Bases"] += len(fastq_fd.next())
                    counts[fastq_file]["Reads"] += 1
                    fastq_fd.next()
                    fastq_fd.next()

                # to take into account "\n" at the end of each line
                counts[fastq_file]["Bases"] = counts[fastq_file]["Bases"] - counts[fastq_file]["Reads"]

        counts.write()

        if stat_file:
            counts.write(stat_file)



#!/usr/bin/env python
__author__ = 'mahajrod'

import numpy as np
from ete2 import Tree, TreeStyle, AttrFace, faces


class HeaderCAFE(list):
    def __str__(self):
        return "#" + "\t".join(self)


class MetadataCAFE():
    def __init__(self, format1, format2):
        self.format1 = format1
        self.format2 = format2


class RecordCAFE():
    def __init__(self, id, tree, family_p_value):
        self.id = id
        self.tree = tree
        self.family_p_value = family_p_value

    def __str__(self):
        return "%s\t%f\t%s" % (self.id, self.family_p_value,
                               self.tree.write(format=100,
                                               features=self.tree.features - set(["support", "name", "dist"]),
                                               format_root_node=True))

    def node_str(self):
        node_string = "%s\t%f" % (self.id, self.family_p_value)
        features = self.tree.features - set(["support", "name", "dist"])
        for node in self.tree.traverse():
            list_of_features = [eval("node.%s" % feature) for feature in features]
            list_of_features = map(lambda x: str(x) if x is not None else ".", list_of_features)
            node_string += "\t" + ",".join(list_of_features)
        return node_string


class GeneralDataCAFE():
    def __init__(self, tree, lambda_value=None):
        self.tree = tree
        for node in self.tree.traverse():
            node.img_style["size"] = 0
        self.lambda_value = lambda_value
        self.node_id_list = [node.id for node in self.tree.traverse()]

    def __str__(self):
        tree_str = "##Common tree\t%s\n" % self.tree.write(format=8,
                                                           features=self.tree.features - set(["support", "name"]),
                                                           format_root_node=True)
        lambda_str = "##Lambda\t%f" % self.lambda_value
        return tree_str + lambda_str

    def draw(self, out_file_prefix, w=140, units="mm"):
        def layout(node, feature):
            if node.up is not None or feature == "id":
                attr = AttrFace(feature, fsize=5, fgcolor="green")
                faces.add_face_to_node(attr, node, 0, position="branch-top")
        for feature in self.tree.features - set(["support", "name"]):
            if feature[0] == "_":
                continue            # skip _nid feature and other technical features of tree

            def layout_arg(node):
                return layout(node, feature)
            ts = TreeStyle()
            ts.layout_fn = layout_arg
            self.tree.render("%s_%s.png" % (out_file_prefix, feature), w=w, units=units, tree_style=ts)

    def draw_expansion_contraction(self, outfile_prefix="expansion_contraction_tree"):

        tree = self.tree
        for node in tree.traverse():
            node.img_style["size"] = 0

        def layout(node):
            if node.up is not None:
                attr = AttrFace("expansion", fsize=7, fgcolor="green", text_prefix="+")
                faces.add_face_to_node(attr, node, 0, position="branch-top")
                attr = AttrFace("decrease", fsize=7, fgcolor="red", text_prefix="-")
                faces.add_face_to_node(attr, node, 0, position="branch-bottom")
            if node.is_leaf():
                attr = AttrFace("name", fsize=10, fgcolor="black", text_prefix="  ", fstyle="italic")
                faces.add_face_to_node(attr, node, 0, position="aligned")
        ts = TreeStyle()
        ts.layout_fn = layout
        ts.optimal_scale_level = "full"
        ts.branch_vertical_margin = 10
        ts.show_leaf_name = False
        #ts.allow_face_overlap =True
        tree.render("%s.png" % outfile_prefix, w=200, units='mm', tree_style=ts, dpi=1200)
        tree.render("%s.svg" % outfile_prefix, w=200, units='mm', tree_style=ts, dpi=1200)

    def draw_significant_expansion_contraction(self, outfile_prefix="significant_expansion_contraction_tree"):

        tree = self.tree
        for node in tree.traverse():
            node.img_style["size"] = 0

        def layout(node):
            if node.up is not None:
                attr = AttrFace("significant_expansion", fsize=7, fgcolor="green", text_prefix="+")
                faces.add_face_to_node(attr, node, 0, position="branch-top")

                attr = AttrFace("significant_contraction", fsize=7, fgcolor="red", text_prefix="-")
                faces.add_face_to_node(attr, node, 0, position="branch-bottom")

            if node.is_leaf():
                attr = AttrFace("name", fsize=10, fgcolor="black", text_prefix="  ", fstyle="italic")
                faces.add_face_to_node(attr, node, 0, position="aligned")
        ts = TreeStyle()
        ts.layout_fn = layout
        ts.optimal_scale_level = "full"
        ts.branch_vertical_margin = 10
        ts.show_leaf_name = False
        #ts.allow_face_overlap =True
        tree.render("%s.png" % outfile_prefix, w=200, units='mm', tree_style=ts, dpi=1200)
        tree.render("%s.svg" % outfile_prefix, w=200, units='mm', tree_style=ts, dpi=1200)

    def write_general_tree(self, out_file):
        with open(out_file, "w") as out_fd:
            out_fd.write(self.tree.write(format=8,
                                         features=self.tree.features - set(["support", "name"]),
                                         format_root_node=True))
            out_fd.write("\n")


class ReportCAFE():
    def __init__(self, report_file=None, from_file=True, records=None, metadata=None, general_data=None, header=None):
        if from_file:
            with open(report_file, "r") as report_fd:
                common_tree = Tree(report_fd.readline().strip()[5:] + ";")
                lambda_value = float(report_fd.readline().strip().split(":")[-1])
                id_tree = Tree(report_fd.readline().strip().split(":")[-1] + ";", format=8)
                for id_tree_node, common_tree_node in zip(id_tree.traverse(), common_tree.traverse()):
                    node_id = int(id_tree_node.name.split("<")[-1][:-1])
                    common_tree_node.add_feature("id", node_id)
                data_format1 = self.split_format1(report_fd.readline().strip().split(":")[-1])
                data_format2 = report_fd.readline().strip().split(":")[-1].split()
                data_format2[0] = data_format2[0][1:-1]
                for i in range(1, len(data_format2)):
                    data_format2[i] = data_format2[i][:-1]
                data_format2 = map(lambda x: int(x), data_format2)
                for data_type in [float, int, int, int]:
                    name, data = report_fd.readline().strip().split(":")
                    data = self.split_format1(data, data_type=data_type)
                    name = name.strip().replace(" ", "_").lower()
                    for node in common_tree.traverse():
                        node.add_feature(name, data[data_format1.index(node.id)] if node.id in data_format1 else None)

                self.general_data = GeneralDataCAFE(common_tree, lambda_value=lambda_value)
                self.metadata = MetadataCAFE(data_format1, data_format2)
                self.header = HeaderCAFE(report_fd.readline().strip().split("\t"))
                self.records = []
                for line in report_fd:
                    id, tree, p_value, vitebri_p_value = line.strip().split()
                    p_value = float(p_value)
                    #print(tree)
                    tree = Tree(tree + ";", format=1) # was 8
                    #print(vitebri_p_value)
                    vitebri_p_value = self.split_format1(vitebri_p_value[1:-1], separator=",", data_type=float)
                    for tree_node, id_node in zip(tree.traverse(), self.general_data.tree.traverse()):
                        if tree_node.is_leaf:
                            tree_node.name, number_of_genes = tree_node.name.split("_")
                        else:
                            number_of_genes = tree_node.name[1:]
                        #print (node.name)
                        number_of_genes = int(number_of_genes)
                        tree_node.add_feature("N", number_of_genes)
                        tree_node.add_feature("p_value", vitebri_p_value[data_format1.index(id_node.id)] if id_node.id in data_format1 else None)
                        parent_tree_node = tree_node.up
                        if parent_tree_node is None:
                            delta = None
                        else:
                            delta = tree_node.N - parent_tree_node.N
                        tree_node.add_feature("delta", delta)

                    self.records.append(RecordCAFE(id, tree, p_value))
        else:
            self.general_data = general_data
            self.metadata = metadata
            self.header = header
            self.records = records

    def __iter__(self):
        for record in self.records:
            yield record

    def split_format1(self, format1_string, separator=None, data_type=int):

        tmp_format1 = format1_string.replace(")", "").replace("(", "").split(separator)
        format1 = []
        #print tmp_format1
        for i in range(0, len(tmp_format1)):
            format1 += map(lambda x: None if x == '-' else data_type(x), tmp_format1[i].split(","))
        return format1

    def convert(self, out_file, output_type="tree"):
        # ouput_type - possible values: tree, node
        with open(out_file, "w") as out_fd:
            out_fd.write(str(self.general_data))
            out_fd.write("\n")
            if output_type == "tree":
                header = "#id\tfamily p-value\ttree\n"
                out_fd.write(header)
                for record in self:
                    out_fd.write(str(record))
                    out_fd.write("\n")
            elif output_type == "node":
                out_fd.write("#Node_format: %s\n" % (",".join(self.records[0].tree.features - set(["support", "name", "dist"]))))
                header = "#id\tfamily_p_value"
                for node in self.general_data.tree.traverse():
                    header += "\t%s" % node.id
                header += "\n"
                out_fd.write(header)
                for record in self:
                    out_fd.write(record.node_str())
                    out_fd.write("\n")

    def get_per_node_values(self, filter_by_p_value=False, p_value_cutoff=0.05):
        node_values_dict = dict((node_id, []) for node_id in self.general_data.node_id_list)
        feature_names_list = ["family_id"] + [feature for feature in self.records[0].tree.features - set(["support", "name", "dist"])]
        for record in self:
            node_index = 0
            for node in record.tree.traverse():
                if not filter_by_p_value or node.p_value <= p_value_cutoff:
                    node_values_dict[self.general_data.node_id_list[node_index]].append([record.id] + [eval("node.%s" % feature) for feature in node.features - set(["support", "name", "dist"])])
                node_index += 1
        return node_values_dict, feature_names_list

    #----------------------------General filtering functions--------------------------------------
    def filter_records(self, expression_function):
        #expression_function must return  boolean value
        filtered = []
        filtered_out = []
        for record in self:
            if expression_function(record):
                filtered.append(record)
            else:
                filtered_out.append(record)
        return filtered, filtered_out

    def filter(self, expression_function):
        filtered_records, filtered_out_records = self.filter_records(expression_function)

        return ReportCAFE(from_file=False, records=filtered_records, general_data=self.general_data,
                          metadata=self.metadata, header=self.header), \
               ReportCAFE(from_file=False, records=filtered_out_records, general_data=self.general_data,
                          metadata=self.metadata, header=self.header),
    #---------------------------------------------------------------------------------------------

    #--------------------------Custom filtering functions-----------------------------------------
    def filter_by_family_p_value(self, p_value_cutoff):
        return self.filter(lambda record: record.family_p_value <= p_value_cutoff)


if __name__ == "__main__":
    cafe_report_file = "/home/mahajrod/genetics/Dobrzhansky/project/gene_families/data/pashas/resultfilereport.cafe"
    cafe_report = ReportCAFE(cafe_report_file)
    #print(cafe_report.general_data.tree.write(features=["id", "remain", "Expansion", "decrease", "average_expansion"]))
    #print(cafe_report.general_data.lambda_value)
    #print(cafe_report.records[3].tree.write(features=["N", "delta", "p_value"], format=8, format_root_node=True))

    cafe_report.general_data.tree.render("mytree.png", w=183, units="mm")
    cafe_report.general_data.draw("labeled_tree")



__author__ = 'mahajrod'

from RouToolPa.Tools.Annotation.Emapper import Emapper
from RouToolPa.Tools.Annotation.VEP import VEP
from RouToolPa.Tools.Annotation.BUSCO import BUSCO
from RouToolPa.Tools.Annotation.SNPeff import SNPeff
from RouToolPa.Tools.Annotation.Barrnap import Barrnap
from RouToolPa.Tools.Annotation.AUGUSTUS import AUGUSTUS
from RouToolPa.Tools.Annotation.Exonerate import Exonerate
from RouToolPa.Tools.Annotation.TransDecoder import TransDecoder


SNPeff_path = ""
SNPeff = SNPeff(jar_path=SNPeff_path)

VEP = VEP()
BUSCO = BUSCO()
Barrnap = Barrnap()
Emapper = Emapper()
AUGUSTUS = AUGUSTUS()
Exonerate = Exonerate()
TransDecoder = TransDecoder()



#!/usr/bin/env python
__author__ = 'Sergei F. Kliver'
import os
import sys

import multiprocessing as mp

from collections import OrderedDict

#from scipy.stats import chisqprob
from scipy import stats
chisqprob = lambda chisq, df: stats.chi2.sf(chisq, df)


#import scipy.stats.distributions.chi2.sf as chisqprob
from statsmodels.sandbox.stats.multicomp import fdrcorrection0

from RouToolPa.Tools.Abstract import Tool
from Parsers.PAML import CodeMLReport
from RouToolPa.Routines import FileRoutines

from RouToolPa.Collections.General import TwoLvlDict

print_mutex = mp.Lock()


def extract_trees_from_codeml_report(list_of_options):
    # this function is global because of stutid damned pickle mode in python!!!!!
    # use mutex to safe write to stdout from multiple threads
    # list of options = [sample_directory, tree_file, report_suffix, output_prefix]
    #sys.stdout.write("Handling %s\n" % list_of_options[0])
    #print list_of_options
    sample_name = list_of_options[0].split("/")
    sample_name = sample_name[-1] if sample_name[-1] != "" else sample_name[-2]
    work_dir = os.getcwd()
    sample_dir = os.path.abspath(list_of_options[0])
    #print sample_dir
    list_of_files = os.listdir(sample_dir)
    report_files_list = []
    suffix_length = len(list_of_options[2])
    for filename in list_of_files:
        if len(filename) < suffix_length:
            continue
        if filename[-suffix_length:] == list_of_options[2]:
            report_files_list.append(filename)

    print_string = "Handling %s\n" % list_of_options[0]
    #print "XXXX"

    os.chdir(sample_dir)
    codeml_report = CodeMLReport(report_files_list[0], treefile=list_of_options[1])

    error_code = 0
    if not report_files_list:
        print_string += "\tNo report file with suffix %s were found\n\tSkipping\n" % list_of_options[2]
        error_code = -1
    elif len(report_files_list) > 1:
        print_string += "\tWere found several (%i) report files with suffix %s\n\tSkipping\n" % (len(report_files_list),
                                                                                                 list_of_options[2])
        error_code = -2
    elif (codeml_report.dNtree is None) or (codeml_report.dStree is None) or (codeml_report.Wtree is None):
        print_string += "\tCodeML report is not full\n\tSkipping\n"
        error_code = -3

    print_mutex.acquire()
    sys.stdout.write(print_string)
    print_mutex.release()
    if error_code < 0:
        return sample_name, error_code

    codeml_report.write_trees(list_of_options[3])
    codeml_report.get_all_values(list_of_options[3] + ".all.values")
    codeml_report.get_feature_values(mode="leaves")
    os.chdir(work_dir)

    tmp = codeml_report.find_leaves_with_positive_selection()
    #if tmp:
    #    print sample_name, tmp
    extract_trees_from_codeml_report.queue.put((sample_name, codeml_report.find_leaves_with_positive_selection()))
    return sample_name, codeml_report.find_leaves_with_positive_selection()


def extract_trees_from_codeml_report_init(queue):
    extract_trees_from_codeml_report.queue = queue


def results_extraction_listener(queue, output_file_prefix, selected_species_list=None):
    """listens for messages on the queue, writes to file."""

    positive_selection_dict = TwoLvlDict()
    selected_species_positive_selection_dict = TwoLvlDict()
    error_fd = open("errors.err", "w")
    error_fd.write("#sample\terror_code\n")
    while 1:
        result = queue.get()
        if isinstance(result[1], int):
            error_fd.write("%s\t%i\n" % (result[0], result[1]))
            continue
        if result == 'finish':
            positive_selection_dict.write("%s.all" % output_file_prefix, absent_symbol=".")
            if selected_species_list:
                selected_species_positive_selection_dict.write("%s.selected_species" % output_file_prefix,
                                                               absent_symbol=".")
            # print positive_selection_dict.table_form(absent_symbol=".")
            break
        if result[1]:
            positive_selection_dict[result[0]] = result[1]
            if selected_species_list:
                for species in selected_species_list:
                    if species in result[1]:
                        if result[0] not in selected_species_positive_selection_dict:
                            selected_species_positive_selection_dict[result[0]] = {}
                        selected_species_positive_selection_dict[result[0]][species] = result[1][species]


class Codeml(Tool):
    def __init__(self, path="", max_threads=4):
        Tool.__init__(self, "codeml", path=path, max_threads=max_threads)
        self.example = """
        treefile = ../tree_no_support.nwk
        seqfile = ../merged_alignment.phy
        outfile = model_0           * main result file name
        noisy = 3  * 0,1,2,3,9: how much rubbish on the screen
        verbose = 1  * 0: concise; 1: detailed, 2: too much
        runmode = 0

        seqtype = 1  * 1:codons; 2:AAs; 3:codons-->AAs
        CodonFreq = 2  * 0:1/61 each, 1:F1X4, 2:F3X4, 3:codon table
        clock = 0
        aaDist = 1  * 0:equal, +:geometric; -:linear, 1-6:G1974,Miyata,c,p,v,a
        model = 0
                   * models for codons:
                       * 0:one, 1:b, 2:2 or more dN/dS ratios for branches
                   * models for AAs or codon-translated AAs:
                       * 0:poisson, 1:proportional, 2:Empirical, 3:Empirical+F
                       * 6:FromCodon, 7:AAClasses, 8:REVaa_0, 9:REVaa(nr=189)
        NSsites = 0
        icode = 1  * 0:universal code; 1:mammalian mt; 2-10:see below
        fix_kappa = 0
        kappa = 5
        fix_omega = 0
        omega = 0.2

        getSE = 0
        RateAncestor = 0

        Small_Diff = .5e-5
        cleandata = 1  * remove sites with ambiguity data (1:yes, 0:no)?
        method = 0   * 0: simultaneous; 1: one branch at a time
        """

    def write_example_ctl(self, example_file):
        with open(example_file, "w") as example_fd:
            example_fd.write(self.example)

    def print_example_ctl(self):
        print(self.example)

    @staticmethod
    def generate_ctl_file(seq_file, tree_file, out_file, ctl_file, seq_type="codons", codon_frequency="F3X4", noisy=3,
                          verbose="concise", runmode=0, clock=0, aminoacid_distance=None, model=1, nssites=0,
                          genetic_code=0, fix_kappa=False, kappa=5, fix_omega=False, omega=0.2, fix_alpha=True, alpha=0,
                          getSE=0, RateAncestor=0, small_difference=0.00001, clean_data=True, method=0, Mgene=None):

        if verbose == "concise":
            verb = 0
        elif verbose == "detailed":
            verb = 1
        elif verbose == "much":
            verb = 2
        else:
            verb = 0

        if seq_type == "codons":
            seq_t = 1
        elif seq_type == "aminoacids":
            seq_t = 2
        elif seq_type == "translate":
            seq_t = 3
        else:
            raise ValueError("Unknown type of sequence was set.")

        if codon_frequency == "equal":
            codon_freq = 0
        elif codon_frequency == "F1X4":
            codon_freq = 1
        elif codon_frequency == "F3X4":
            codon_freq = 2
        elif codon_frequency == "table":
            codon_freq = 3
        else:
            raise ValueError("Codon frequency was not recognized")

        options = "treefile = %s\n" % tree_file
        options += "seqfile = %s\n" % seq_file
        options += "outfile = %s\n" % out_file
        options += "noisy = %i  * 0,1,2,3,9: how much rubbish on the screen\n" % noisy
        options += "verbose = %i  * 0: concise; 1: detailed, 2: too much\n" % verb
        options += "runmode = %i * 0: for this mode only unrooted tree is allowed; 1,2: only rooted tree\n\n" % runmode

        options += "seqtype = %i  * 1:codons; 2:AAs; 3:codons-->AAs\n" % seq_t
        options += "CodonFreq = %i  * 0:1/61 each, 1:F1X4, 2:F3X4, 3:codon table\n" % codon_freq
        options += "clock = %s\n" % clock
        options += "aaDist = %i  * 0:equal, +:geometric; -:linear, 1-6:G1974,Miyata,c,p,v,a\n" % aminoacid_distance if aminoacid_distance else ""
        options += """model = %i
                   * models for codons:
                       * 0:one, 1:b, 2:2 or more dN/dS ratios for branches
                   * models for AAs or codon-translated AAs:
                       * 0:poisson, 1:proportional, 2:Empirical, 3:Empirical+F
                       * 6:FromCodon, 7:AAClasses, 8:REVaa_0, 9:REVaa(nr=189)\n""" % model
        options += "NSsites = %i\n" % nssites
        options += "icode = %i  * 0:universal code; 1:mammalian mt; 2-10:see below\n" % genetic_code
        options += "fix_kappa = %i\n" % (1 if fix_kappa else 0)
        options += "kappa = %i\n" % kappa
        options += "fix_omega = %i\n" % (1 if fix_omega else 0)
        options += "omega = %f\n\n" % omega

        options += "fix_alpha = %i\n" % (1 if fix_alpha else 0)
        options += "alpha = %f\n" % alpha


        options += "getSE = %i\n" % getSE
        options += "RateAncestor = %i\n" % RateAncestor

        options += "Small_Diff = %f\n" % small_difference
        options += "cleandata = %i  * remove sites with ambiguity data (1:yes, 0:no)?\n" % (1 if clean_data else 0)
        options += "method = %i   * 0: simultaneous; 1: one branch at a time\n" % method
        options += ("""Mgene = %i * is used only with G option in sequence file.
                   * 0: same k and pi, different cs(proportional branch lengths between genes)
                   * 1: different k, pi and unproportional branch length
                   * 2: same k, different pi and cs
                   * 3: same pi, different k and cs
                   * 4: different k, pi, cs(proportional branch length)\n""" % Mgene) if Mgene else ""

        with open(ctl_file, "w") as ctl_fd:
            ctl_fd.write(options)

    def run_codeml(self, ctl_file):

        options = " %s" % ctl_file

        self.execute(options=options)

    def parallel_codeml(self, in_dir, tree_file, out_dir, seq_type="codons", codon_frequency="F3X4", noisy=0,
                          verbose="concise", runmode=0, clock=0, aminoacid_distance=None, model=1, nssites=0,
                          genetic_code=0, fix_kappa=False, kappa=5, fix_omega=False, omega=0.2, getSE=0, RateAncestor=0,
                          small_difference=0.000001, clean_data=True, method=0, Mgene=None):

        FileRoutines.safe_mkdir(out_dir)
        alignment_files_list = FileRoutines.make_list_of_path_to_files(in_dir)
        tree_file_abs_path = os.path.abspath(tree_file)
        options_list = []
        dir_list = []
        for filename in alignment_files_list:
            directory, basename, extension = FileRoutines.split_filename(filename)
            filename_out_dir = os.path.abspath("%s/%s/" % (out_dir, basename))
            out_file = "%s/%s.out" % (filename_out_dir, basename)
            ctl_file = "%s/%s.ctl" % (filename_out_dir, basename)

            options_list.append(ctl_file)
            dir_list.append(filename_out_dir)
            FileRoutines.safe_mkdir(filename_out_dir)
            self.generate_ctl_file(os.path.abspath(filename), tree_file_abs_path, out_file, ctl_file, seq_type=seq_type,
                                   codon_frequency=codon_frequency, noisy=noisy, verbose=verbose, runmode=runmode,
                                   clock=clock, aminoacid_distance=aminoacid_distance, model=model, nssites=nssites,
                                   genetic_code=genetic_code, fix_kappa=fix_kappa, kappa=kappa, fix_omega=fix_omega,
                                   omega=omega, getSE=getSE, RateAncestor=RateAncestor, Mgene=Mgene,
                                   small_difference=small_difference, clean_data=clean_data, method=method)
        self.parallel_execute(options_list, dir_list=dir_list)

    def parallel_results_extraction(self, in_dir, tree_file, report_suffix, output_prefix,
                                    report_file_prefix, selected_species_list=None):
        work_dir = os.getcwd()
        input_directory = os.path.abspath(in_dir)
        dir_list = sorted(os.listdir(input_directory))
        tree_file_abs_path = os.path.abspath(tree_file)
        options_list = []

        for filename in dir_list:
            sample_dir = "%s/%s/" % (input_directory, filename)
            if os.path.isdir(sample_dir):
                options_list.append([sample_dir, tree_file_abs_path, report_suffix, output_prefix])

        os.chdir(input_directory)

        manager = mp.Manager()
        queue = manager.Queue()
        process_pool = mp.Pool(self.threads + 1, extract_trees_from_codeml_report_init, [queue])
        watcher = process_pool.apply_async(results_extraction_listener,
                                           (queue, report_file_prefix, selected_species_list))

        #print options_list
        process_pool.map(extract_trees_from_codeml_report, options_list)
        queue.put('finish')
        process_pool.close()
        os.chdir(work_dir)

    def parallel_positive_selection_test(self, in_dir, tree_file, out_dir, results_file, seq_type="codons", codon_frequency="F3X4", noisy=3,
                                         verbose="concise", runmode=0, clock=0, aminoacid_distance=None,
                                         genetic_code=0, fix_kappa=False, kappa=5,  getSE=0, RateAncestor=0,
                                         small_difference=0.000001, clean_data=True, method=0):
        """
        This function implements positive selection test (branch-site model)
        for branch labeled in tree file using model_A vs model_A_null(omega fixed to 1) comparison
        """

        FileRoutines.safe_mkdir(out_dir)
        alignment_files_list = FileRoutines.make_list_of_path_to_files(in_dir)
        tree_file_abs_path = os.path.abspath(tree_file)
        options_list = []
        dir_list = []
        basename_dir_list = []
        model_list = ["Model_A", "Model_A_null"]
        fix_omega_dict = {"Model_A": False, "Model_A_null": True}

        for filename in alignment_files_list:
            directory, basename, extension = FileRoutines.split_filename(filename)
            filename_out_dir = os.path.abspath("%s/%s/" % (out_dir, basename))
            basename_dir_list.append(basename)
            FileRoutines.safe_mkdir(filename_out_dir)

            for model in model_list:
                model_dir = "%s/%s/" % (filename_out_dir, model)
                FileRoutines.safe_mkdir(model_dir)
                out_file = "%s/%s/%s.out" % (filename_out_dir, model, basename)
                ctl_file = "%s/%s/%s.ctl" % (filename_out_dir, model, basename)

                options_list.append("%s.ctl" % basename)
                dir_list.append(model_dir)

                self.generate_ctl_file(os.path.abspath(filename), tree_file_abs_path, out_file, ctl_file,
                                       seq_type=seq_type, codon_frequency=codon_frequency, noisy=noisy, verbose=verbose,
                                       runmode=runmode, clock=clock, aminoacid_distance=aminoacid_distance, model=2,
                                       nssites=2, genetic_code=genetic_code, fix_kappa=fix_kappa,
                                       kappa=kappa, fix_omega=fix_omega_dict[model], omega=1, getSE=getSE,
                                       RateAncestor=RateAncestor, Mgene=0,
                                       small_difference=small_difference, clean_data=clean_data, method=method)

        self.parallel_execute(options_list, dir_list=dir_list)

        results_dict = OrderedDict()
        double_delta_dict = OrderedDict()
        raw_pvalues_dict = OrderedDict()
        raw_pvalues_list = []

        for basename in basename_dir_list:
            results_dict[basename] = OrderedDict()
            for model in model_list:
                output_file = "%s/%s/%s/%s.out" % (out_dir, basename, model, basename)
                codeml_report = CodeMLReport(output_file)
                results_dict[basename][model] = codeml_report.LnL

        skipped_genes_set = set()
        for basename in basename_dir_list:
            for model in model_list:
                if results_dict[basename][model] is None:
                    print("LnL was not calculated for %s" % basename)
                    skipped_genes_set.add(basename)
                    break
            else:
                doubled_delta = 2 * (results_dict[basename]["Model_A"] - results_dict[basename]["Model_A_null"])
                p_value = chisqprob(doubled_delta, 1)  # degrees of freedom = 1

                double_delta_dict[basename] = doubled_delta
                raw_pvalues_dict[basename] = p_value
                raw_pvalues_list.append(p_value)

        adjusted_pvalues_list = fdrcorrection0(raw_pvalues_list)[1]
        #print adjusted_pvalues_list
        i = 0
        with open(results_file, "w") as out_fd:
            out_fd.write("id\tmodel_a_null,LnL\tmodel_a,LnL\t2*delta\traw p-value\tadjusted p-value\n")
            for basename in basename_dir_list:
                for model in model_list:
                    if results_dict[basename][model] is None:
                        print("LnL was not calculated for %s" % basename)
                        break
                else:
                    #doubled_delta = 2 * (results_dict[basename]["Model_A"] - results_dict[basename]["Model_A_null"])
                    #p_value = chisqprob(doubled_delta, 1) # degrees of freedom = 1

                    #print basename, results_dict[basename]["Model_A_null"],results_dict[basename]["Model_A"], double_delta_dict[basename], raw_pvalues_dict[basename], adjusted_pvalues_list[i]

                    out_fd.write("%s\t%f\t%f\t%f\t%f\t%f\n" % (basename, results_dict[basename]["Model_A_null"],
                                                           results_dict[basename]["Model_A"], double_delta_dict[basename],
                                                           raw_pvalues_dict[basename], adjusted_pvalues_list[i]))
                    i += 1




#!/usr/bin/env python
__author__ = 'mahajrod'
from collections import OrderedDict


#TODO: think about parsing data in modules - at moment all data are counted as strings
#tested with FastQC v0.10.1
class FastQCModule():
    def __init__(self, name, status, header_list, data, info_list=None):
        self.name = name
        self.status = status
        self.header_list = header_list
        self.data = data
        self.info_list = info_list

    def __str__(self):
        string = ">>%s\t%s\n" % (self.name, self.status)
        string += ("#" + "\t".join(self.info_list) + "\n") if self.info_list else ""
        string += "#" + "\t".join(self.header_list) + "\n"
        keys = list(self.data.keys())
        for i in range(0, len(self.data[keys[0]])):
            entry = []
            for key in keys:
                entry.append(str(self.data[key][i]))
            string += "\t".join(entry) + "\n"

        string += ">>END_MODULE\n"
        return string


class FastQCReport():
    def __init__(self, fastqc_report_file):
        with open(fastqc_report_file, "r") as in_fd:
            self.version = in_fd.readline().strip().split("\t")[1]
            self.report = OrderedDict({})
            for line in in_fd:
                temp = line.strip()
                if temp[0:2] == ">>":
                    module_name, status = temp[2:].split("\t")
                    self.report[module_name] = self.init_module(in_fd, module_name, status)

    def init_module(self, fd, name, status):
        info_list = fd.readline().strip()[1:].split("\t") if name == "Sequence Duplication Levels" else None
        header_list = fd.readline().strip()[1:].split("\t")
        data = OrderedDict({})
        for entry in header_list:
            data[entry] = []
        for line in fd:
            if line[:2] == ">>":
                return FastQCModule(name, status, header_list, data, info_list=info_list)
            temp = line.strip().split("\t")
            for (key, value) in zip(header_list, temp):
                data[key].append(value)

    def overrepresented_sequences(self):
        return self.report["Overrepresented sequences"].data["Sequence"]

    def overrepresented_kmers(self):
        return self.report["Kmer Content"].data["Sequence"]

if __name__ == "__main__":
    FastQC_file = FastQCReport("../example_data/FastQC/fastqc_data.txt")
    print(FastQC_file.overrepresented_sequences())
    print(FastQC_file.overrepresented_kmers())